#include "about.h"

// This provides an "about" widget
About::About(QWidget *parent, QString version) : QWidget(parent) {

	// The position and dimensions of the widget shown&hidden
	QDesktopWidget desk;
	rectShown = QRect((desk.screenGeometry().width()-900)/2.0,(desk.screenGeometry().height()-550)/2.0,900,550);
	rectHidden = QRect(rectShown.x(),-rectShown.height(),rectShown.width(),rectShown.height());
	this->setGeometry(rectHidden);

	// This boolean holds whether the widget is shown or hidden
	isShown = false;

	// This boolean may block the widget in certain circumstances
	blockAll = false;

	// Style the widget
	this->setObjectName("about");
	this->setStyleSheet("QWidget#about { background: rgba(0,0,0,200); border-top-left-radius: 20px; border-bottom-left-radius: 20px; }");


	// Create and set the scrollarea with main layout
	QVBoxLayout *central = new QVBoxLayout;
	scroll = new QScrollArea(this);
	scroll->setStyleSheet("background: transparent");
	QWidget *scrollWidget = new QWidget(scroll->widget());
	scrollWidget->setStyleSheet("background: transparent");
	scroll->setWidgetResizable(true);
	scroll->setSizePolicy(QSizePolicy::Expanding,QSizePolicy::Expanding);
	scrollWidget->setLayout(central);
	scroll->setWidget(scrollWidget);
	QVBoxLayout *scCentral = new QVBoxLayout;
	this->setLayout(scCentral);
	scCentral->addWidget(scroll);

	// The string right at the top, giving the license and author information
	QLabel *license = new QLabel("<div style=\"font-size: 8pt\">" + tr("Photo %1, by Lukas Spies, 2012 (photoQt@ymail.com)").arg(version) + " &ndash; web: photoqt.co.cc &ndash; " + tr("Licensed under GPL, without any guarantee") + "</div>");
	license->setWordWrap(true);
	license->setStyleSheet("color: white");
	central->addWidget(license);

	// The logo (Thanks to Archie Arevalo)
	QLabel *logo = new QLabel;
	logo->setPixmap(QPixmap(":/img/splash.png"));
	QHBoxLayout *logoLay = new QHBoxLayout;
	logo->setStyleSheet("background: transparent");
	logoLay->addStretch();
	logoLay->addWidget(logo);
	logoLay->addStretch();
	central->addLayout(logoLay);

	// The main text block
	QString txt = tr("Photo is a simple image viewer, designed to be good looking, highly configureable, yet easy to use and fast.") +  "<br><br>";
	txt += tr("I myself was born and raised in Germany and am now a student of Mathematics @ NUI Galway, Ireland. This project simply developed out of my search for something like this, but finding nothing.") + "<br><br>";
	txt += tr("There are a lot of great and amazing image viewers out there, but with Photo I want to go a new way, making it possible to view images <i>\"the nice way\"</i>.") + "<br><br>";
	txt += tr("That said, I hope you find this program useful!<br><br>If you want to contribute to Photo by translating it into another language (currently English, German and Greek available), drop me an email (photoQt@ymail.com) or leave a comment at the qt-apps.org project page.") + "<br><br>";
	txt += "<b>" + tr("Also don't forget to check out the brand-new website of Photo: PhotoQt.co.cc") + "</b><br><br><br>";
	txt += tr("If you find a bug or if you have a question or suggestion, tell me. I'm open to any feedback I get :)") + "<br><br>";
	txt += "<h3>" + tr("Thanks to:") + "</h3>";
	txt += "- <i>Archie Arevalo</i>: " + tr("Logo") + "<br>";
	txt += "- " + tr("<i>Everybody</i> who packaged Photo for their distribution") + "<br>";
	txt += "<h3>" + tr("Translations:") + "</h3>";
	txt += "- <i>Lukas Spies</i>: " + tr("English, German") + "<br>";
	txt += "- <i>Dimitrios Glentadakis</i>: " + tr("Greek") + "<br>";
	txt += "- <i>Tomas Bartl</i>: " + tr("Czech") +"<br>";
	txt += "- <i>Daniel Korbel</i>: " + tr("Polish") + "<br>";
	txt += "- <i>Vincenzo Cerminara</i>: " + tr("Italian") + "<br>";
	txt += QString::fromUtf8("- <i>Mladen Pejaković</i>: ") + tr("Serbian") + "<br>";
	txt += "- <i>Manuel E. Gutierrez</i>: " + tr("Spanish") + "<br>";
	//: The "...and you?" question serves as a rethorical question, hopefully motivating some users to make another translation
	txt += tr("...and you?") + "<br><br><br>";

	QLabel *text = new QLabel(txt);
	text->setStyleSheet("color: white");
	text->setWordWrap(true);
	central->addWidget(text);
	central->addStretch();

	// Initiate the animation class
	ani = new QPropertyAnimation(this,"geometry");

	// A close button at the bottom
	QPushButton *close = new QPushButton(tr("Close"));
	close->setStyleSheet("color: white; border: 1px solid white; border-radius: 10px; padding: 5 15 5 15; background:rgba(0,0,0,50)");
	QHBoxLayout *closeLay = new QHBoxLayout;
	closeLay->addStretch();
	closeLay->addWidget(close);
	closeLay->addStretch();
	scCentral->addLayout(closeLay);
	connect(close, SIGNAL(clicked()), this, SLOT(animate()));

	this->show();

}

// Animate open the widget
void About::animate() {

//	QDesktopWidget desk;
//	rectShown = QRect((desk.width()-900)/2.0,(desk.height()+13-550)/2.0,900,550);
//	rectHidden = QRect(rectShown.x(),-rectShown.height(),rectShown.width(),rectShown.height());

	// Widget can only be opened if it's not blocked
	if(!blockAll) {

		if(ani->state() == QPropertyAnimation::Stopped && isShown == false) {

			// Reset the scroll position
			scroll->verticalScrollBar()->setValue(0);

			isShown = true;
			ani->setDuration(200);
			qDebug() << "Showing About Widget";
			ani->setStartValue(rectHidden);
			ani->setEndValue(rectShown);
			ani->setEasingCurve(QEasingCurve::InBack);
			ani->start();
			emit block(true);

		}
	}

	// Hide the widget again
	if(isShown && ani->state() == QPropertyAnimation::Stopped) {
		isShown= false;

		ani->setDuration(200);
		qDebug() << "Hiding About Widget";
		ani->setStartValue(rectShown);
		ani->setEndValue(rectHidden);
		ani->setEasingCurve(QEasingCurve::OutBack);
		ani->start();
		emit block(false);

	}

}



// Enabling styling of widget
void About::paintEvent(QPaintEvent *) {
	QStyleOption o;
	o.initFrom(this);
	QPainter p(this);
	style()->drawPrimitive(QStyle::PE_Widget, &o, &p, this);
}


About::~About() { }
