#include "exifdata.h"

// This class provides a widget for displaying meta (Exif) information
Exif::Exif(QWidget *parent): QWidget(parent) {

	// style widget
	this->setStyleSheet("border-radius: 8px; background-color: rgba(20, 20, 20, 180);");
	this->setContentsMargins(10,10,10,10);

	onlineservice = "maps.google.com";

	// Initiate and set layout
	central = new QVBoxLayout;
	this->setLayout(central);

	// Setup animation
	ani = new QPropertyAnimation(this,"geometry");

	// A boolean storing if widget is shown or hidden
	isShown = false;

	// block widget in certain circumstances
	blockAll = false;

	// Store if widget shall be triggered my mouse movement
	mouseTrickerEnable = true;

	// The dimensions of the widget (shown and hidden)
	rectShown = QRect(50,50,400,600);
	rectHidden = QRect(-rectShown.width(),rectShown.y(),rectShown.width(),rectShown.height());

	// Initially the widget it hidden
	this->setGeometry(rectHidden);

	// These two labels are displayed at the bottom for toggling state of widget staying open or not
	noOpen = tr("Doesn't Stay Open (Click here to change)");
	yesOpen = tr("Stays Open (Click here to change)");

	// This widget is shown when no file is loaded
	empty = new QLabel("<center><h1>" + tr("No file loaded") + "</h1></center>");
	empty->setStyleSheet("background: transparent; color: rgb(135,135,135);");
	empty->setWordWrap(true);
	central->addWidget(empty);

	// Setup the labels
	setupLabels();


	// These QMaps hold the translation for some fields
	mapAll.clear();

	// The light source "translated" into a human-readable format
	QMap<QString,QString> mapLightSource;
	//: This string refers to the light source
	mapLightSource.insert("0",tr("Unknown"));
	//: This string refers to the light source
	mapLightSource.insert("1",tr("Daylight"));
	//: This string refers to the light source
	mapLightSource.insert("2",tr("Fluorescent"));
	//: This string refers to the light source
	mapLightSource.insert("3",tr("Tungsten (incandescent light)"));
	//: This string refers to the light source
	mapLightSource.insert("4",tr("Flash"));
	//: This string refers to the light source
	mapLightSource.insert("9",tr("Fine weather"));
	//: This string refers to the light source
	mapLightSource.insert("10",tr("Cloudy Weather"));
	//: This string refers to the light source
	mapLightSource.insert("11",tr("Shade"));
	//: This string refers to the light source
	mapLightSource.insert("12",tr("Daylight fluorescent (D 5700 - 7100K)"));
	//: This string refers to the light source
	mapLightSource.insert("13",tr("Day white fluorescent (N 4600 - 5400K)"));
	//: This string refers to the light source
	mapLightSource.insert("14",tr("Cool white fluorescent") + "(W 3900 - 4500K)");
	//: This string refers to the light source
	mapLightSource.insert("15",tr("White fluorescent") + "(WW 3200 - 3700K)");
	//: This string refers to the light source
	mapLightSource.insert("17",tr("Standard light") + " A");
	mapLightSource.insert("18",tr("Standard light") + " B");
	mapLightSource.insert("19",tr("Standard light") + " C");
	mapLightSource.insert("20","D55");
	mapLightSource.insert("21","D65");
	mapLightSource.insert("22","D75");
	mapLightSource.insert("23","D50");
	//: This string refers to the light source
	mapLightSource.insert("24",tr("ISO studio tungsten"));
	//: This string refers to the light source
	mapLightSource.insert("255",tr("Other light source"));

	//: This string identifies that flash was fired
	QString fYes = tr("yes");
	//: This string identifies that flash wasn't fired
	QString fNo = tr("no");
	//: This string refers to a flash mode
	QString fNoStrobe = tr("strobe return light not detected");
	//: This string refers to a flash mode
	QString fYesStrobe = tr("strobe return light detected");
	//: This string refers to a flash mode
	QString fComp = tr("compulsory flash mode");
	//: This string refers to a flash mode
	QString fAuto = tr("auto mode");
	//: This string refers to a flash mode
	QString fRed = tr("red-eye reduction mode");
	//: This string refers to a flash mode
	QString fYesReturn = tr("return light detected");
	//: This string refers to a flash mode
	QString fNoReturn = tr("return light not detected");

	// The flash "translated" into a human-readable format
	QMap<QString,QString> mapFlash;
	mapFlash.insert("0",fYes);
	mapFlash.insert("1",fNo);
	mapFlash.insert("5",fNoStrobe);
	mapFlash.insert("6",fYesStrobe);
	mapFlash.insert("9",fYes + " (" + fComp + ")");
	mapFlash.insert("13",fYes + " (" + fComp + ", " + fNoReturn + ")");
	mapFlash.insert("15",fYes + " (" + fComp + ", " + fYesReturn + ")");
	mapFlash.insert("16",fNo + " (" + fComp + ")");
	mapFlash.insert("24",fNo + " (" + fAuto + ")");
	mapFlash.insert("25",fYes + " (" + fAuto + ")");
	mapFlash.insert("29",fYes + " (" + fAuto + ", " + fNoReturn + ")");
	mapFlash.insert("31",fYes + " (" + fAuto + ", " + fYesReturn + ")");
	//: This string refers to a flash mode
	mapFlash.insert("32",tr("No flash function"));
	mapFlash.insert("65",fYes + " (" + fRed + ")");
	mapFlash.insert("69",fYes + " (" + fRed + ", " + fNoReturn + ")");
	mapFlash.insert("71",fYes + " (" + fRed + ", " + fYesReturn + ")");
	mapFlash.insert("73",fYes + " (" + fComp + ", " + fRed + ")");
	mapFlash.insert("77",fYes + " (" + fComp + ", " + fRed + ", " + fNoReturn + ")");
	mapFlash.insert("79",fYes + " (" + fComp + ", " + fRed + ", " + fYesReturn + ")");
	mapFlash.insert("89",fYes + " (" + fAuto + ", " + fRed + ")");
	mapFlash.insert("93",fYes + " (" + fAuto + ", " + fNoReturn + ", " + fRed + ")");
	mapFlash.insert("95",fYes + " (" + fAuto + ", " + fYesReturn + ", " + fRed + ")");

	// The scene type "translated" into a human-readable format
	QMap<QString,QString> mapSceneType;
	//: This string refers to a scene type
	mapSceneType.insert("0",tr("Standard"));
	//: This string refers to a scene type
	mapSceneType.insert("1",tr("Landscape"));
	//: This string refers to a scene type
	mapSceneType.insert("2",tr("Portrait"));
	//: This string refers to a scene type
	mapSceneType.insert("3",tr("Night Scene"));

	// Store these maps in a global map with the internal ids as accessor
	mapAll.insert("lightsource",mapLightSource);
	mapAll.insert("flash",mapFlash);
	mapAll.insert("scenetype",mapSceneType);


}

// Setup the labels
void Exif::setupLabels() {

	// This QMap stores the exiv2 key and the connected internal ids
	keyVal.clear();

	// Initiate the labels
	labels << tr("Filename");
	labelsId << "filename";

	labels << tr("Filetype");
	labelsId << "filetype";

	labels << tr("Filesize");
	labelsId << "filesize";

	labels << tr("Dimensions");
	labelsId << "dimensions";

	labels << "";
	labelsId << "";

	labels << tr("Make");
	labelsId << "make";
	keyVal.insert("Exif.Image.Make","make");

	labels << tr("Model");
	labelsId << "model";
	keyVal.insert("Exif.Image.Model","model");

	labels << tr("Software");
	labelsId << "software";
	keyVal.insert("Exif.Image.Software","software");

	labels << tr("Time photo was taken");
	labelsId << "phototaken";
	keyVal.insert("Exif.Photo.DateTimeOriginal","phototaken");

	labels << "";
	labelsId << "";

	labels << tr("Exposure Time");
	labelsId << "exposuretime";
	keyVal.insert("Exif.Photo.ExposureTime","exposuretime");
	units.insert("exposuretime","1 s");

	labels << tr("Flash");
	labelsId << "flash";
	keyVal.insert("Exif.Photo.Flash","flash");

	labels << tr("ISO");
	labelsId << "iso";
	keyVal.insert("Exif.Photo.ISOSpeedRatings","iso");

	labels << "";
	labelsId << "";

	labels << tr("Scene Type");
	labelsId << "scenetype";
	keyVal.insert("Exif.Photo.SceneCaptureType","scenetype");

	labels << tr("Focal Length");
	labelsId << "flength";
	keyVal.insert("Exif.Photo.FocalLength","flength");
	units.insert("flength","1 mm");

	labels << tr("F Number");
	labelsId << "fnumber";
	keyVal.insert("Exif.Photo.FNumber","fnumber");
	units.insert("fnumber","0F");

	labels << tr("Light Source");
	labelsId << "lightsource";
	keyVal.insert("Exif.Photo.LightSource","lightsource");

	labels << "";
	labelsId << "";

	labels << tr("GPS Position");
	labelsId << "gps";

	// The GPS position is composed of these 4 data sets
	keyVal.insert("Exif.GPSInfo.GPSLatitudeRef","gpslatref");
	keyVal.insert("Exif.GPSInfo.GPSLatitude","gpslat");
	keyVal.insert("Exif.GPSInfo.GPSLongitudeRef","gpslonref");
	keyVal.insert("Exif.GPSInfo.GPSLongitude","gpslon");

	// This QMap stores the visibility state of the labels
	showItems.clear();

	// Eventually set up the labels
	for(int i = 0; i < labels.length(); ++i) {

		if(labels.at(i) != "") {
			if(labelsId.at(i) != "gps") {
				QLabel *l = new QLabel("<b>" + labels.at(i) + "</b>");
				l->setStyleSheet("color: white");
				items.insert(labelsId.at(i),l);
				central->addWidget(l);
				showItems.insert(labelsId.at(i),true);
				l->hide();
			} else {
				LabelClick *l = new LabelClick;
				l->setText("<b>" + labels.at(i) + "</b>");
				connect(l, SIGNAL(clicked()), this, SLOT(gpsClick()));
				l->setStyleSheet("color: white");
				items.insert(labelsId.at(i),l);
				central->addWidget(l);
				showItems.insert(labelsId.at(i),true);
				l->hide();
			}

		} else
			central->addSpacing(8);

	}

	central->addSpacing(8);

	// This pushbutton enabled an option for triggering "stay open" state
	stay = new QPushButton(noOpen);
	stay->setStyleSheet("color: white; font-size: 7pt; background: transparent; padding: 5px");
	QHBoxLayout *stayLay = new QHBoxLayout;
	stayLay->addStretch();
	stayLay->addWidget(stay);
	stayLay->addStretch();
	central->addLayout(stayLay);
	stay->hide();

	connect(stay, SIGNAL(clicked()), this, SLOT(changeStayOpenState()));

}

// Update the labels data
void Exif::updateData(QString currentfile, QSize origSize, bool exiv2Supported) {

	if(currentfile != "--start-in-tray") {

	// This button is only shown when mouse triggering is enabled
	if(mouseTrickerEnable)
		stay->show();
	else
		stay->hide();

	// As soon as data is loaded, the empty label is hidden
	empty->hide();

	// Setup some data
	QFileInfo info(currentfile);
	QImageReader file(currentfile);

	// This string is re-used again and again
	QString temp = "";

	// Set the filename and show label
	temp = items["filename"]->text();
	if(items["filename"]->text().contains(":"))
		temp = items["filename"]->text().split(":").at(0);
	items["filename"]->setText(temp + ": " + info.fileName());
	items["filename"]->setToolTip(items["filename"]->text());
	if(showItems["filename"])
		items["filename"]->show();
	else
		items["filename"]->hide();

	// Set the filetype and show label
	temp = items["filetype"]->text();
	if(items["filetype"]->text().contains(":"))
		temp = items["filetype"]->text().split(":").at(0);
	items["filetype"]->setText(temp + ": " + file.format());
	items["filetype"]->setToolTip(items["filetype"]->text());
	if(showItems["filetype"])
		items["filetype"]->show();
	else
		items["filetype"]->hide();

	// Set the filesize and show label
	temp = items["filesize"]->text();
	if(items["filesize"]->text().contains(":"))
		temp = items["filesize"]->text().split(":").at(0);
	items["filesize"]->setText(temp + ": " + QString("%1").arg(info.size()/1024) + " KB");
	items["filesize"]->setToolTip(items["filesize"]->text());
	if(showItems["filesize"])
		items["filesize"]->show();
	else
		items["filesize"]->hide();

	// This stringlist holds the found option. If an option is not available it is marked as this
	QStringList gotTheseKeys;

	// The 4 different gps data sets (composed into one string at the end)
	QString gpsLonRef = "";
	QString gpsLon = "";
	QString gpsLatRef = "";
	QString gpsLat = "";

	// If exiv2 supports this file format
	if(exiv2Supported) {

		// Access and iterate through the exif data
		Exiv2::Image::AutoPtr image = Exiv2::ImageFactory::open(currentfile.toUtf8().constData());
		image->readMetadata();
		Exiv2::ExifData &exifData = image->exifData();
		Exiv2::ExifData::const_iterator end = exifData.end();

		for(Exiv2::ExifData::const_iterator it = exifData.begin(); it != end; ++it) {

			// Store the value and the key
			QString val = QString::fromStdString(Exiv2::toString(it->value().toString()));
			QString key = QString::fromStdString(it->key());

			// Here we detect the orientation stored in the exif data
			if(key == "Exif.Image.Orientation") {

				// 1 = No rotation/flipping
				if(val == "1")
					emit setOrientation(0);
				// 2 = Horizontally Flipped
				else if(val == "2")
					emit setOrientation(-360);
				// 3 = Rotated by 180 degrees
				else if(val == "3")
					emit setOrientation(180);
				// 4 = Rotated by 180 degrees and flipped horizontally
				else if(val == "4")
					emit setOrientation(-180);
				// 5 = Rotated by 270 degrees and flipped horizontally
				else if(val == "5")
					emit setOrientation(-270);
				// 6 = Rotated by 270 degrees
				else if(val == "6")
					emit setOrientation(270);
				// 7 = Flipped Horizontally and Rotated by 90 degrees
				else if(val == "7")
					emit setOrientation(-90);
				// 8 = Rotated by 90 degrees
				else if(val == "8")
					emit setOrientation(90);
			}

			// If the data is wanted
			if(keyVal.keys().contains(key)) {

				// Store the unit for this data (set up in setupLabels() function)
				QString unit1 = "";
				QString unit2 = "";
				temp = units.value(keyVal[key]);
				if(temp != "") {
					if(temp.startsWith("1"))
						unit2 = temp.remove(0,1);
					else
						unit1 = temp.remove(0,1);
				}

				// If the data is supposed to be "translatedable" into human-readable format, then do it
				if(mapAll.contains(keyVal[key])) {
					QMap<QString,QString> m = mapAll[keyVal[key]];
					if(m.keys().contains(val))
						val = m[val];
					// If data is in unknown format
					else
						val = "[" + tr("Unknown data") + "]";
				}

				// The exposure time is converted into 1/x format (sometimes it's stored in 10/x or other formats)
				if(keyVal[key] == "exposuretime" && val.contains("/")) {
					QStringList split = val.split("/");
					if(split.at(0) != "1") {
						int t1 = split.at(0).toInt();
						float t2 = split.at(1).toFloat();
						// I got a bug report of Photo crashing for certain images that have an exposure time of "0/1". So we have to check for it, or we get a division by zero, i.e. crash
						if(t1 == 0) {
							t1 = 0;
							t2 = 0;
							val = "0";
						} else if(t1 != 1) {
							t1 = t1/t1;
							t2 = t2/t1;
							val = QString("%1/%2").arg(t1).arg(t2);
						} else {
							val = QString("%1/%2").arg(t1).arg(t2);
						}
					}

				}

				// Convert the focal length into a float number
				if((keyVal[key] == "flength" || keyVal[key] == "fnumber") && val.contains("/")) {
					QStringList split = val.split("/");
					float t1 = split.at(0).toFloat();
					float t2 = split.at(1).toFloat();
					t1 = t1/t2;
					val = QString("%1").arg(t1);
				}

				// Format the data string
				if(keyVal[key] == "phototaken") {
					QStringList split = val.split(" ");
					QStringList split2 = split.at(0).split(":");
					val = split2.at(2) + "/" + split2.at(1) + "/" + split2.at(0) + ", " + split.at(1);
				}

				// The four gps data sets are stored seperately (composed into one string later-on)
				if(keyVal[key] == "gpslonref") {
					gpsLonRef = val;
				} else if(keyVal[key] == "gpslon") {
					gpsLon = val;
				} else if(keyVal[key] == "gpslatref") {
					gpsLatRef = val;
				} else if(keyVal[key] == "gpslat") {
					gpsLat = val;

				// All the "normal" data
				} else {

					temp = items[keyVal[key]]->text();
					if(items[keyVal[key]]->text().contains(":"))
						temp = items[keyVal[key]]->text().split(":").at(0);
					items[keyVal[key]]->setText(temp + ": " + unit1 + val + unit2);
					items[keyVal[key]]->setToolTip(items[keyVal[key]]->text());
					if(showItems[keyVal[key]])
						items[keyVal[key]]->show();
					else
						items[keyVal[key]]->hide();
					gotTheseKeys << keyVal[key];

				}
			}
		}

	}

	// If gps data is available
	if(gpsLon != "") {

		// Format the latitude string
		QStringList split = gpsLat.split(" ");
		for(int i = 0; i < split.length(); ++i) {
			if(split.at(i).contains("/")) {
				float t1 = split.at(i).split("/").at(0).toFloat();
				float t2 = split.at(i).split("/").at(1).toFloat();
				split.replace(i,QString("%1").arg(t1/t2));
			}

		}
		gpsLat = split.at(0) + "&deg;" + split.at(1) + "'" + split.at(2) + "''";

		// Format the longitude string
		split = gpsLon.split(" ");
		for(int i = 0; i < split.length(); ++i) {
			if(split.at(i).contains("/")) {
				float t1 = split.at(i).split("/").at(0).toFloat();
				float t2 = split.at(i).split("/").at(1).toFloat();
				split.replace(i,QString("%1").arg(t1/t2));
			}
		}
		gpsLon = split.at(0) + "&deg;" + split.at(1) + "'" + split.at(2) + "''";

		// Compose all the gps data into one string
		QString gps = gpsLat + " " + gpsLatRef + ", " + gpsLon + " " + gpsLonRef;

		temp = items["gps"]->text();
		if(items["gps"]->text().contains(":"))
			temp = items["gps"]->text().split(":").at(0);
		items["gps"]->setText(temp + ": " + gps);
		items["gps"]->setToolTip(items["gps"]->text() + "<br>" + tr("(Click to open position on online map)"));
		gotTheseKeys << "gps";
		if(showItems["gps"])
			items["gps"]->show();
		else
			items["gps"]->hide();
	}

	// Set the dimensions (could possibly be read from exif data as well, but easier to get from the file itself (transmitted by mainwindow.cpp)
	temp = items["dimensions"]->text();
	if(items["dimensions"]->text().contains(":"))
		temp = items["dimensions"]->text().split(":").at(0);
	items["dimensions"]->setText(temp + QString(": %1x%2 px").arg(origSize.width()).arg(origSize.height()));
	items["dimensions"]->setToolTip(items["dimensions"]->text());
	if(showItems["dimensions"])
		items["dimensions"]->show();
	else
		items["dimensions"]->hide();
	gotTheseKeys << "dimensions";

	// All the exif data that wasn't set is marked as that
	for(int i = 0; i < labelsId.length(); ++i) {
		QStringList ignore;
		ignore << "" << "filename" << "filetype" << "filesize";
		if(!gotTheseKeys.contains(labelsId.at(i)) && !ignore.contains(labelsId.at(i))) {
			temp = items[labelsId.at(i)]->text();
			if(items[labelsId.at(i)]->text().contains(":"))
				temp = items[labelsId.at(i)]->text().split(":").at(0);
			QString unavailable = tr("not set");
			// If file format us unsupported, tell the user
			if(!exiv2Supported)
				unavailable = tr("unsupported format");
			items[labelsId.at(i)]->setText(temp + ": <i>[" + unavailable + "]</i>");
			items[labelsId.at(i)]->setToolTip(items[labelsId.at(i)]->text());
			if(showItems[labelsId.at(i)])
				items[labelsId.at(i)]->show();
			else
				items[labelsId.at(i)]->hide();
		}
	}

	}

}

// Animate open/close the widget
void Exif::animate(int x, int y, bool forceClose, bool forceOpen) {

	// If widget is not blocked
	if(!blockAll) {

		// Open the widget
		if(forceOpen || (mouseTrickerEnable && x >= 0 && x < 40 && y >= rectShown.y() && y <= rectShown.y()+rectShown.height() && !forceClose && !isShown && ani->state() == QPropertyAnimation::Stopped)) {
			isShown = true;

			ani->setDuration(400);
			qDebug() << "Showing Meta Info Widget";
			ani->setStartValue(rectHidden);
			ani->setEndValue(rectShown);
			ani->setEasingCurve(QEasingCurve::InBack);
			ani->start();

		}
	}

	// Close the widget
	if((forceClose || ((x > rectShown.x()+rectShown.width() || y < rectShown.y() || y > rectShown.y()+rectShown.height()) && mouseTrickerEnable)) && isShown && ani->state() == QPropertyAnimation::Stopped && !staying) {
		isShown= false;

		ani->setDuration(300);
		qDebug() << "Hiding Meta Info Widget";
		ani->setStartValue(rectShown);
		ani->setEndValue(rectHidden);
		ani->setEasingCurve(QEasingCurve::OutBack);
		ani->start();

	}

}


// Change the open state (i.e. adjust text and boolean)
void Exif::changeStayOpenState() {

	if(!blockAll) {

		if(staying) {
			stay->setText(noOpen);
			staying = false;
		} else {
			stay->setText(yesOpen);
			staying = true;
		}

	}

}

// Adjust the height of the widget
void Exif::adjustHeight() {

	// The height depends on the amount of labels shown
	int count = 0;
	for(int i = 0; i < labelsId.length(); ++i) {
		if(showItems[labelsId.at(i)])
			++count;
	}

	// Adjust the QRects
	rectShown = QRect(rectShown.x(),rectShown.y(),rectShown.width(),count*35);
	rectHidden = QRect(-rectShown.width(),rectShown.y(),rectShown.width(),rectShown.height());

	// Set the appropriate QRect
	if(isShown)
		this->setGeometry(rectShown);
	else if(!isShown)
		this->setGeometry(rectHidden);


}

void Exif::gpsClick() {

	QStringList temp = items["gps"]->text().split(":");
	temp.removeFirst();
//	qDebug() << "Click on GPS:" << temp.join(":");
	QString loc = temp.join(":").trimmed();

	if(!loc.contains("[") && !loc.contains("]")) {

	//	loc.replace(" ","+");
		loc.replace("&deg;",QString::fromUtf8("°"));
	//	loc.replace("'","%27");

		QUrl url;
		qDebug() << "ONLINESERVICE:" << onlineservice;
		if(onlineservice == "bing.com/maps")
			url.setUrl("http://www.bing.com/maps/?sty=b&q=" + loc);
		else
			url.setUrl("http://maps.google.com/maps?t=h&q=" + loc);


		if(!QDesktopServices::openUrl(QUrl(url)))
			qDebug() << "Couldn't open URL...";

	}

}


// Make the widget styleable
void Exif::paintEvent(QPaintEvent *) {
	QStyleOption o;
	o.initFrom(this);
	QPainter p(this);
	style()->drawPrimitive(QStyle::PE_Widget, &o, &p, this);
}


Exif::~Exif() { }



void LabelClick::mouseReleaseEvent(QMouseEvent *) {
	emit clicked();
}
