#ifndef EXIFDATA_H
#define EXIFDATA_H

#include <QWidget>
#include <QStyleOption>
#include <QPainter>
#include <QVBoxLayout>
#include <QLabel>
#include <QImageReader>
#include <QFileInfo>
#include <QDateTime>
#include <QPropertyAnimation>
#include <QtDebug>
#include <QPushButton>
#include <QMessageBox>
#include <QMap>
#include <QDesktopServices>
#include <QUrl>

#include "exiv2/image.hpp"

// A custom label (the "x" top right corner) for closing Photo
class LabelClick : public QLabel {
	Q_OBJECT

protected:
	void mouseReleaseEvent(QMouseEvent *e);
signals:
	void clicked();
};

// This class provides a widget for displaying meta (Exif) information
class Exif : public QWidget {

	Q_OBJECT

public:
	Exif(QWidget *parent = 0);
	~Exif();

	// The main layout of the widget
	QVBoxLayout *central;

	// All the labels for displaying exif info
	QMap<QString,QLabel*> items;
	void setupLabels();

	// A label displaying an "Nothing loaded" message
	QLabel *empty;

	// The animation class
	QPropertyAnimation *ani;

	// This boolean stores if the widget is shown or hidden
	bool isShown;

	// The two rects for the two states (hidden/shown)
	QRect rectShown;
	QRect rectHidden;

	// The "stay open" state and button
	bool staying;
	QPushButton *stay;
	QString noOpen;
	QString yesOpen;

	// The blocking boolean
	bool blockAll;

	// All the label data for displaying and detecting data
	QMap<QString,QString> keyVal;
	QStringList labels;
	QStringList labelsId;
	QMap<QString,QString> units;

	// This QMap stores QMaps for "translatin" exif data into a human readable format
	QMap<QString, QMap<QString,QString> > mapAll;

	// Stores whether items are shown
	QMap<QString,bool> showItems;

	// This boolean stores if mouse triggering is en-/disabled
	bool mouseTrickerEnable;

	// The online map service opened when clicked on GPS coordinates in the exif window
	QString onlineservice;

public slots:
	// Update the label data
	void updateData(QString currentfile, QSize origSize, bool exiv2Supported = true);

	// Animate open/close the widget
	void animate(int x, int y, bool forceClose = false, bool forceOpen = false);

	// Changing the "stay open" state
	void changeStayOpenState();

	// Adjusting the widget height
	void adjustHeight();

	// A click on the GPS coordinates
	void gpsClick();

signals:
	// Update orientation of big image
	void setOrientation(int deg);

protected:
	void paintEvent(QPaintEvent *);


};

#endif // EXIFDATA_H
