#include "exifrotconfirm.h"

// This widget asks for confirmation when exif data says: "rotate!"
ExifRotConf::ExifRotConf(QWidget *parent) : QWidget(parent) {

	// We set an object name for use for setting a stylesheet
	this->setObjectName("ask");
	// Style the look
	this->setStyleSheet("QWidget#ask {border: 2px solid black; background-color: rgba(0,0,0,190); border-radius: 20px; } QWidget { color: white; }");

	// A global vars for the rotating degrees
	degree = 0;

	// Central Layout
	QVBoxLayout *central = new QVBoxLayout;
	this->setLayout(central);

	// Setup the two QRects for the widget shown/hidden
	QDesktopWidget desk;
	rectShown = QRect(desk.screenGeometry().width()/2-300,(desk.screenGeometry().height()-200)/2,600,200);
	rectHidden = QRect(rectShown.x(),-rectShown.height(),rectShown.width(),rectShown.height());

	// Initially the widget is hidden
	this->setGeometry(rectHidden);
	isShown = false;

	central->addSpacing(10);

	// Some info text
	QLabel *title = new QLabel("<center><h1>" + tr("Rotate Image?") + "</h1>" + tr("The Exif data of this image says, that this image is supposed to be rotated.") + "<br><b>" + tr("Do yo want to apply rotation?") + "</b>");
	title->setStyleSheet("background:transparent");
	title->setWordWrap(true);

	central->addWidget(title);
	central->addSpacing(10);

	// A checkbox, its value is passed on to mainwindow.cpp
	neverask = new QCheckBox(tr("Don't ask me that again"));

	// The stylesheet for the checkbox (if we'd leave them unchanged, the tick would be nearly impossible to see)
	QString ss = "QCheckBox::indicator:unchecked { image: url(:/img/checkbox_unchecked.png); }";
	ss += "QCheckBox::indicator:checked {image: url(:/img/checkbox_checked.png);}";
	ss += "QCheckBox { background:transparent; }";
	neverask->setStyleSheet(ss);

	QHBoxLayout *neveraskLay = new QHBoxLayout;
	neveraskLay->addStretch();
	neveraskLay->addWidget(neverask);
	neveraskLay->addStretch();
	central->addLayout(neveraskLay);

	// A subtitle right below checkbox informing the user that he can adjust this setting anytime in the settings
	QLabel *subtitle = new QLabel("<center><i>" + tr("Note: This behaviour can be adjusted at anytime in the settings.") + "</i></center>");
	subtitle->setWordWrap(true);
	central->addWidget(subtitle);
	central->addSpacing(10);

	// The two buttons: Yes and No
	ok = new QPushButton(tr("Yep, rotate it"));
	ok->setObjectName("ok");
	QPushButton *notok = new QPushButton(tr("Nope, don't do it"));
	notok->setObjectName("notok");
	ok->setStyleSheet("padding: 5px; color: white; border-radius: 5px; background: black;");
	notok->setStyleSheet("padding: 5px; color: white; border-radius: 5px; background: black;");
	connect(ok, SIGNAL(clicked()), this, SLOT(buttonClicked()));
	connect(notok, SIGNAL(clicked()), this, SLOT(buttonClicked()));

	QHBoxLayout *buttons = new QHBoxLayout;
	buttons->addStretch();
	buttons->addWidget(ok);
	buttons->addWidget(notok);
	buttons->addStretch();
	central->addLayout(buttons);

	central->addStretch();

	// Set up the animation
	ani = new QPropertyAnimation(this,"geometry");
	connect(ani, SIGNAL(finished()), this, SLOT(animationFinished()));

}

// Called from mainwindow.cpp, asking for confirmation
void ExifRotConf::doRotation(int deg) {

	degree = deg;
	animate();

}

// When one of the two button was clicked
void ExifRotConf::buttonClicked() {

	// First start to hide the widget
	animate();

	// Get the button that was clicked
	QPushButton *cl = (QPushButton*) sender();
	if(cl->objectName() == "ok")
		emit rotate(degree,true);
	else
		emit rotate(0,true);

	// If the checkbox is checked, emit it to be stored permanently
	if(neverask->isChecked()) {
		if(cl->objectName() == "ok")
			emit rotDontAsk(true);
		else
			emit rotDontAsk(false);
	}

}

// Animate in/out the widget
void ExifRotConf::animate() {

	if(ani->state() == QPropertyAnimation::Stopped) {

		if(!isShown) {

			isShown = true;
			ani->setDuration(200);
			qDebug() << "Showing Confirmation Widget";
			ani->setStartValue(rectHidden);
			ani->setEndValue(rectShown);
			ani->setEasingCurve(QEasingCurve::InBack);
			ani->start();
			emit block(true);

		// Hide the widget again
		} else {
			isShown= false;

			ani->setDuration(200);
			qDebug() << "Hiding Confirmation Widget";
			ani->setStartValue(rectShown);
			ani->setEndValue(rectHidden);
			ani->setEasingCurve(QEasingCurve::OutBack);
			ani->start();

		}

	}

}

// We unblock the widget not before it completely is hidden (otherwise if the user is too fast in switching to the next img which requires this widget as well, it might not be shown)
void ExifRotConf::animationFinished() {

	if(ani->state() == QPropertyAnimation::Stopped && isShown == false)
		emit block(false);

}



// Enabling styling of widget
void ExifRotConf::paintEvent(QPaintEvent *) {
	QStyleOption o;
	o.initFrom(this);
	QPainter p(this);
	style()->drawPrimitive(QStyle::PE_Widget, &o, &p, this);
}

ExifRotConf::~ExifRotConf() { }
