#include "filehandling.h"

HandleFile::HandleFile(QWidget *parent) : QWidget(parent) {

	// Adjust the look of this widget
	this->setObjectName("widg");
	this->setStyleSheet("QWidget#widg { border: 2px solid black;} *{border-radius: 8px; background-color: rgba(20, 20, 20, 210);}");

	// Set the widget geometry
	QDesktopWidget desk;
	rectShown = QRect(desk.screenGeometry().width()/2-300,desk.screenGeometry().height()/3,600,150);
	rectHidden = QRect(rectShown.x(),-rectShown.height(),rectShown.width(),rectShown.height());
	rectHiddenAcceppted = QRect(rectShown.x(), desk.height(),rectShown.width(),rectShown.height());
	this->setGeometry(rectHidden);

	// Initially the widget is hidden
	isShown = false;

	// Initialize the animation
	ani = new QPropertyAnimation(this,"geometry");

	// This value is set at the beginning of the animation, identifying if the animation is animating the widget in or out
	willBe = false;
	// At the end of the animation the willBe bool is copied to the isShown bool
	connect(ani, SIGNAL(finished()), this, SLOT(handleAniFinished()));

}

// Create the rename layout
void HandleFile::setRename() {

	// Initialize the layout
	rename = new QVBoxLayout(this);

	// The object name, that's how we identify lateron the currently set layout
	rename->setObjectName("rename");

	rename->addStretch();

	// The top title
	QLabel *title = new QLabel("<b>" + tr("Rename File:") + "</b> ",this);
	title->setStyleSheet("background: none; color: white");
	QHBoxLayout *titleLay = new QHBoxLayout;
	titleLay->addWidget(title);
	titleLay->addStretch();
	rename->addLayout(titleLay);
	rename->addSpacing(10);

	// Get the current file info
	QFileInfo info(currentfile);

	// Presenting the current filename, with the actual file basename in a QLineEdit for editing
	QHBoxLayout *renameLay = new QHBoxLayout;
	renamePath = new QLabel(info.absolutePath() + "/");
	renamePath->setStyleSheet("background: none; color: white");
	renameDo = new QLineEdit;
	renameDo->setStyleSheet("color: white;");
	renameDo->setSizePolicy(QSizePolicy::Expanding,QSizePolicy::Preferred);
	renameDo->setText(info.completeBaseName());
	renameDo->setEnabled(false);		// needs to be set to false, or it will cause some problems with the keyboard being caught in it
	renameEnding = new QLabel("." + info.completeSuffix());
	renameEnding->setStyleSheet("background: none; color: white");
	renameLay->addWidget(renamePath);
	renameLay->addWidget(renameDo);
	renameLay->addWidget(renameEnding);

	rename->addLayout(renameLay);

	// The buttons at the bottom, stylish adjusted
	QHBoxLayout *buttons = new QHBoxLayout;
	QPushButton *ok = new QPushButton(tr("Go ahead and rename file"));
	ok->setStyleSheet("padding: 5px; color: white; border-radius: 5px; background: black;");
	QPushButton *cancel = new QPushButton(tr("No, I actually like the old name better"));
	cancel->setStyleSheet("padding: 5px; color: white; border-radius: 5px; background: black;");
	buttons->addStretch();
	buttons->addWidget(ok);
	buttons->addWidget(cancel);
	buttons->addSpacing(20);

	rename->addSpacing(15);
	rename->addLayout(buttons);
	rename->addStretch();

	connect(ok, SIGNAL(clicked()), this, SLOT(renameDoIt()));
	connect(cancel, SIGNAL(clicked()), this, SLOT(animate()));
	connect(renameDo, SIGNAL(returnPressed()), this, SLOT(renameDoIt()));
	renameDo->setEnabled(false);

}

// Create the delete layout
void HandleFile::setDelete() {

	// Initialize the layout
	del = new QVBoxLayout;

	// The object name, that's how we identify lateron the currently set layout
	del->setObjectName("del");

	// The warning intro
	QLabel *txt1 = new QLabel("<center>" + tr("You're attempting to delete the file:") + "</center>");
	txt1->setStyleSheet("background: none; color: white");
	del->addWidget(txt1);

	// This will hold the filename lateron
	deleteFile = new QLabel("<b><center>''</center></b>");
	deleteFile->setStyleSheet("background: none; color: white");
	del->addWidget(deleteFile);

	del->addSpacing(10);

	// The warning ending
	QLabel *txt2 = new QLabel("<center>" + tr("Are you sure you want to continue?") + "<br><br><b>" + tr("This step cannot be reversed!!") + "</b></center>");
	txt2->setStyleSheet("background: none; color: white");

	del->addWidget(txt2);

	del->addStretch();

	// The buttons at the bottom
	QPushButton *ok = new QPushButton(tr("Indeed, I want to delete it"));
	ok->setStyleSheet("padding: 5px; color: white; border-radius: 5px; background: black;");
	QPushButton *cancel = new QPushButton(tr("Oops! No, don't delete it"));
	cancel->setStyleSheet("padding: 5px; color: white; border-radius: 5px; background: black;");

	QHBoxLayout *buttons = new QHBoxLayout;
	buttons->addStretch();
	buttons->addWidget(ok);
	buttons->addWidget(cancel);
	buttons->addStretch();

	del->addLayout(buttons);

	connect(ok, SIGNAL(clicked()), this, SLOT(deleteDoIt()));
	connect(cancel, SIGNAL(clicked()), this, SLOT(animate()));

}

// Doing the actual renaming of a file
void HandleFile::renameDoIt() {

	// The current file info
	QFileInfo info(currentfile);

	// The new filename
	QString baseName = info.absolutePath() + "/" + renameDo->text() + "." + info.completeSuffix();

	if(baseName != currentfile) {

		// Issue warning if target file already exists
		int msg = QMessageBox::Yes;
		if(QFile(baseName).exists())
			msg = QMessageBox::warning(0,tr("Target file exists already"),tr("There exists already a file with this filename. Do you want to overwrite it?"),QMessageBox::Yes | QMessageBox::Default, QMessageBox::No);

		if(msg == QMessageBox::Yes) {

			if(QFile(baseName).exists())
				QFile(baseName).remove();

			// Copy the file and remove old version
			QFile file(currentfile);

			if(file.copy(baseName)) {
				if(!file.remove())
					qDebug() << "ERROR: Couldn't remove old version of file! The file exists now twice!";
			} else
				qDebug() << "ERROR: Couldn't copy file to new filename!!";


			// Re-enable the mainwindow layout
			emit blockAll(false);

			// Tell the mainwindow class that the file was renamed (in order to update the thumbnails, counter, etc.)
			emit fileRenamed(baseName);

			// Hide widget
			animate("",1);

		}

	} else {

		emit blockAll(false);
		animate("",1);

	}

}

// Doing the actual deleting of a file
void HandleFile::deleteDoIt() {

	qDebug() << "Deleting file";

	// Re-enable the mainwindow layout
	emit blockAll(false);
	// Hide the widget
	animate("",1);
	// Tell the mainwindow class, that the file is supposed to be deleted
	emit fileDeleted();

}

// Adjust the current layout (set filename, etc.)
void HandleFile::adjust(QString layout) {

	if(layout == "rename") {

		QFileInfo info(currentfile);

		renamePath->setText(info.absolutePath() + "/");
		renameDo->setText(info.completeBaseName());
		renameEnding->setText("." + info.completeSuffix());

	} else if(layout == "delete") {

		deleteFile->setText("<b><center>" + currentfile + "</center></b>");
		deleteFile->setFocus();

	}

}

// Doing some stuff whenever the widget was animated in/out
void HandleFile::handleAniFinished() {

	// Copy the willBe bool to the isShown bool
	if(willBe) {
		isShown = true;
	} else {
		isShown = false;
		emit blockAll(false);
	}

	// Set the rename file info
	if(this->layout()->objectName() == "rename" && isShown) {
		renameDo->setEnabled(true);
		renameDo->setFocus();
		renameDo->selectAll();
	}

}

// Hide/Show the widget
void HandleFile::animate(QString layout, int success) {

	// If it currently hidden and about to be shown, set the right layout and adjust it
	if(!isShown) {
		if(layout == "rename") {
			qDeleteAll(this->children());
			setRename();
			this->setLayout(rename);
			adjust("rename");
		} else {
			qDeleteAll(this->children());
			setDelete();
			this->setLayout(del);
			adjust("delete");
		}
	}

	// Animate in
	if(ani->state() == QPropertyAnimation::Stopped && isShown == false) {
		ani->setDuration(300);
		qDebug() << "Showing File Handling Widget";
		willBe = true;
		emit blockAll(true);
		ani->setStartValue(rectHidden);
		ani->setEndValue(rectShown);
		ani->setEasingCurve(QEasingCurve::InBack);
		ani->start();
	} else if(isShown && ani->state() == QPropertyAnimation::Stopped) {
		if(this->layout()->objectName() == "rename")
			renameDo->setEnabled(false);
		ani->setDuration(250);
		qDebug() << "Hiding File Handling Widget";
		willBe = false;
		ani->setStartValue(rectShown);
		if(success == 0)
			ani->setEndValue(rectHidden);
		else
			ani->setEndValue(rectHiddenAcceppted);
		ani->setEasingCurve(QEasingCurve::OutBack);
		ani->start();
	}

}

// This way we get the Enter event for deleting a file
void HandleFile::keyReleaseEvent(QKeyEvent *e) {

	if((e->key() == Qt::Key_Enter || e->key() == Qt::Key_Return) && this->layout()->objectName() == "del" && isShown)
		deleteDoIt();
	else
		QWidget::keyReleaseEvent(e);

}

// Make the widget styleable
void HandleFile::paintEvent(QPaintEvent *) {
	QStyleOption o;
	o.initFrom(this);
	QPainter p(this);
	style()->drawPrimitive(QStyle::PE_Widget, &o, &p, this);
}


HandleFile::~HandleFile() { }
