#ifndef FILEHANDLING_H
#define FILEHANDLING_H

#include <QWidget>
#include <QStyleOption>
#include <QPainter>
#include <QDesktopWidget>
#include <QPropertyAnimation>
#include <QtDebug>
#include <QVBoxLayout>
#include <QLabel>
#include <QFileInfo>
#include <QLineEdit>
#include <QPushButton>
#include <QKeyEvent>
#include <QMessageBox>

// A widget for editing/deleting a file
class HandleFile : public QWidget {

	Q_OBJECT

public:
	HandleFile(QWidget *parent = 0);
	~HandleFile();

	// The dimensions of this widget
	QRect rectShown;
	QRect rectHidden;
	QRect rectHiddenAcceppted;

	// We use Qt's property animation
	QPropertyAnimation *ani;

	// This boolean stores the current state of the widget
	bool isShown;

	// This boolean stores at the beginning of an animation what the widget will be; at the end of animation, value is copied to isShown
	bool willBe;

	// The currentfile
	QString currentfile;

	// The two layouts (set via functions below dynaimcally as needed)
	QVBoxLayout *rename;
	QVBoxLayout *del;

	// Rename
	QLabel *renamePath;
	QLineEdit *renameDo;
	QLabel *renameEnding;

	// Delete
	QLabel *deleteFile;

	// Set the layouts
	void setRename();
	void setDelete();

public slots:
	// Do the actual renaming (and the mainwindow class)
	void renameDoIt();

	// Do the actual deletion (and tell mainwindow class)
	void deleteDoIt();

	// Adjust some bits and pieces in the layout (e.g. filename, enabled/disabled, ...)
	void adjust(QString layout);

	// Animate the widget in/out
	void animate(QString layout = "", int success = 0);

	// Called, whenever the animation has finished
	void handleAniFinished();

protected:
	void paintEvent(QPaintEvent *);
	void keyReleaseEvent(QKeyEvent *);

signals:
	void fileRenamed(QString newName);
	void fileDeleted();
	void blockAll(bool);

};

#endif // FILEHANDLING_H
