#include <QtGui/QApplication>
#include <QTranslator>
#include "mainwindow.h"


int main(int argc, char *argv[]) {

	// This string holds the current version
	QString version = "0.8";

	// A help message for the command line
	QString hlp = "\nPhoto v" + version + " - created by Lukas Spies (photoQt@ymail.com) - License: GPL\n";
	hlp += "Photo is a fast, simple, good looking, yet powerfull and highly configurable image viewer.\n\n";

	hlp += "Usage: photo [options] [file]\n\n";

	hlp += "Options:\n";
	hlp += "\t--h, --help\t\tThis help message\n";
	hlp += "\t--open\t\tOpens the open file dialog (also shows Photo if hidden)\n\n";

	hlp += "\t--toggle\t\tToggles Photo - hides Photo if visible, shows if hidden\n";
	hlp += "\t--show\t\t\tShows Photo (does nothing if already shown)\n";
	hlp += "\t--hide\t\t\tHides Photo (does nothing if already hidden)\n\n";

	hlp += "\t--start-in-tray\t\tStarts Photo and hides it into the system tray\n";
	hlp += "\t\t\t\tThis command is useful e.g. for putting into the autostart. This way you have Photo\n";
	hlp += "\t\t\t\talways available in an instant whenever you click on an image file.\n\n";

	hlp += "   Enjoy Photo :-)\n\n\n";

	// This file is updated by a running instance of Photo every 500 milliseconds - check
	QFile chk(QDir::homePath() + "/.photo/running");
	QString all = "";
	if(chk.open(QIODevice::ReadOnly)) {
		QTextStream in(&chk);
		all = in.readAll();
	}

	// If photo was started with "--h" or "--help", show help message
	if(argc == 2 && (QString(argv[1]) == "--help" || QString(argv[1]) == "--h")) {

		std::cout << hlp.toStdString();

		return 0;

	// If an instance of photo is running, we check for command line arguments
	} else if(QDateTime::currentMSecsSinceEpoch() - all.toLongLong() < qint64(520)) {

		// This is the content of the file used to communicate with running photo instance
		QString cont = "";

		// This boolean is set to true if an unknown command is used
		bool err = false;


		for(int i = 1; i < argc; ++i) {

			// Check for the command line options
			if(QString(argv[i]) == "--toggle")
				cont += "toggle\n";
			else if(QString(argv[i]) == "--open")
				cont += "open\n";
			else if(QString(argv[i]) == "--show")
				cont += "show\n";
			else if(QString(argv[i]) == "--hide")
				cont += "hide\n";
			else if(QString(argv[i]).startsWith("-"))	// If an unknown command is used, set boolean to true
				err = true;
			else {
				cont += argv[i];
				if(!cont.startsWith("/"))
					cont = QFileInfo(cont).absoluteFilePath();
			}

		}

		// If photo is called without any arguments, "show" is used
		if(argc == 1)
			cont = "show";

		// If only correct arguments were used
		if(!err) {
			// Write the commands into this file, which is checked regularly by a running instance of photo
			QFile f(QDir::homePath() + "/.photo/cmd");
			f.remove();
			if(f.open(QIODevice::WriteOnly)) {
				QTextStream out(&f);
				out << cont;
				f.close();
			} else
				qDebug() << "ERROR! Couldn't write to file '~/.photo/cmd'. Unable to communicate with running process";

		// If an uncorrect argument was used
		} else
			std::cout << hlp.toStdString();

		return 0;

	// If photo isn't running already and a command line argument (besides possible filename and "--start-in-tray"), show help message
	} else if(argc >= 2 && QString(argv[1]).startsWith("-") && QString(argv[1]) != "--start-in-tray") {

		std::cout << hlp.toStdString();

		return 0;

	// If photo isn't running and no command line argument (besides filename and "--start-in.tray") was used
	} else {

		// This boolean stores if photo needs to be minimized to the tray
		bool startintray = false;

		// Check if this option is set
		if(QString(argv[1]) == "--start-in-tray" || QString(argv[2]) == "--start-in-tray")
			startintray = true;

		// If photo is supposed to be started miniized in system tray
		if(startintray) {
			// If the option "Use Tray Icon" in the settings is not set, we set it
			QFile set(QDir::homePath() + "/.photo/settings");
			if(set.open(QIODevice::ReadOnly)) {
				QTextStream in(&set);
				QString all = in.readAll();
				if(all.contains("TrayIcon=0")) {
					all.replace("TrayIcon=0","Trayicon=1");
					set.close();
					set.remove();
					set.open(QIODevice::WriteOnly);
					QTextStream out(&set);
					out << all;
					set.close();
				} else
					set.close();
			} else
				qDebug() << "Unable to ensure TrayIcon is enabled - make sure it is enabled!!";
		}

		// Initialize the application
		QApplication a(argc, argv);

		// The translator
		QTranslator trans;

		// Check if languaged is changed to one of the possible translations
		QFile fileSettings(QDir::homePath() + "/.photo/settings");
		if(fileSettings.open(QIODevice::ReadOnly)) {
			QTextStream in(&fileSettings);
			QString all = in.readAll();
			if(all.contains("Language=de")) {
				trans.load(":/lang/photo_de");
				a.installTranslator(&trans);
				qDebug() << "Load German Translation";
			} else if(all.contains("Language=el")) {
				trans.load(":/lang/photo_el");
				a.installTranslator(&trans);
				qDebug() << "Load Greek Translation";
			} else if(all.contains("Language=cz")){
				trans.load(":/lang/photo_cz");
				a.installTranslator(&trans);
				qDebug() << "Load Czech Translation";
			} else if(all.contains("Language=pl")) {
				trans.load(":/lang/photo_pl");
				a.installTranslator(&trans);
				qDebug() << "Load Polish Translation";
			} else if(all.contains("Language=es")) {
				trans.load(":/lang/photo_es");
				a.installTranslator(&trans);
				qDebug() << "Load Spanish Translation";
			} else if(all.contains("Language=it")) {
				trans.load(":/lang/photo_it");
				a.installTranslator(&trans);
				qDebug() << "Load Italian Translation";
			} else if(all.contains("Language=sr_RS")) {
				trans.load(":/lang/photo_sr_RS");
				a.installTranslator(&trans);
				qDebug() << "Load Serbian Ekavian Translation";
			} else if(all.contains("Language=sr_BA")) {
				trans.load(":/lang/photo_sr_BA");
				a.installTranslator(&trans);
				qDebug() << "Load Serbian Ijekavian Translation";
			}
			fileSettings.close();
		} else
			qDebug() << "ERROR! Couldn't read settings file.";


		// Ensure that the config folder exists
		QDir dir(QDir::homePath() + "/.photo");
		if(!dir.exists())
			dir.mkdir(QDir::homePath() + "/.photo");

		// Check if the settings file exists. If not, create a file with default settings
		QFile file(QDir::homePath() + "/.photo/settings");
		if(!file.exists()) {
			if(!file.open(QIODevice::WriteOnly))
				qDebug() << "ERROR: Couldn't write settings file! Please ensure that you have read&write access to home directory";
			else {
				qDebug() << "Creating basic settings file";
				QTextStream out(&file);
				out << "Version=" + version + "\n";
				file.close();
			}

		// If file does exist, check if it is from a previous version -> photo was updated
		} else {
			if(!file.open(QIODevice::ReadWrite))
				qDebug() << "ERROR: Couldn't read settings file! Please ensure that you have read&write access to home directory";
			else {
				QTextStream in(&file);
				QString all = in.readAll();

				// If it contains no "Version=" (photo 0.1)
				if(!all.contains("Version=")) {
					file.close();
					file.remove();
					file.open(QIODevice::ReadWrite);
					in << "Version=" + version  + "\n" << all;
				// If it doesn't contain current version (some previous version)
				} else if(!all.contains("Version=" + version)) {
					file.close();
					file.remove();
					file.open(QIODevice::ReadWrite);
					QStringList allSplit = all.split("\n");
					allSplit.removeFirst();
					QString allFile = "Version=" + version + "\n" + allSplit.join("\n");
					in << allFile;
				}

				file.close();

			}
		}

		// Check of thumbnail database exists. If not, create it
		QFile database(QDir::homePath() + "/.photo/thumbnails");
		if(!database.exists()) {

			qDebug() << "Create Database";

			QSqlDatabase db = QSqlDatabase::addDatabase("QSQLITE");
			db.setDatabaseName(QDir::homePath() + "/.photo/thumbnails");
			db.open();
			QSqlQuery query;
			query.exec("CREATE TABLE Thumbnails (filepath TEXT,thumbnail BLOB, filelastmod INT, thumbcreated INT)");
			if(query.lastError().text().trimmed().length())
				qDebug() << "ERROR (Creating Thumbnail Datbase):" << query.lastError().text().trimmed();
			query.clear();
			db.close();


		}

		// This int holds 1 if photo was updated and 2 if it's newly installed
		int update = 0;

		// Check for shortcuts configuration
		QFile sh(QDir::homePath() + "/.photo/shortcuts");
		if(!sh.exists()) {
			if(sh.open(QIODevice::WriteOnly)) {
				QTextStream out(&sh);
				QString defShortcuts = "";
				defShortcuts += "Version=" + version + "\n";
				defShortcuts += "0::O::__openOld\n";
				defShortcuts += "0::Right::__next\n";
				defShortcuts += "0::Space::__next\n";
				defShortcuts += "0::Left::__prev\n";
				defShortcuts += "0::Backspace::__prev\n";
				defShortcuts += "0::+::__zoomIn\n";
				defShortcuts += "0::Ctrl++::__zoomIn\n";
				defShortcuts += "0::-::__zoomOut\n";
				defShortcuts += "0::Ctrl+-::__zoomOut\n";
				defShortcuts += "0::0::__zoomReset\n";
				defShortcuts += "0::Ctrl+1::__zoomActual\n";
				defShortcuts += "0::R::__rotateR\n";
				defShortcuts += "0::L::__rotateL\n";
				defShortcuts += "0::Ctrl+0::__rotate0\n";
				defShortcuts += "0::Ctrl+H::__flipH\n";
				defShortcuts += "0::Ctrl+V::__flipV\n";
				defShortcuts += "0::Ctrl+E::__hideMeta\n";
				defShortcuts += "0::E::__settings\n";
				defShortcuts += "0::F2::__rename\n";
				defShortcuts += "0::Ctrl+C::__copy\n";
				defShortcuts += "0::Ctrl+M::__move\n";
				defShortcuts += "0::Delete::__delete\n";
				defShortcuts += "0::I::__about\n";
				defShortcuts += "0::H::__noThumb\n";
				defShortcuts += "0::S::__stopThb\n";
				defShortcuts += "0::Ctrl+R::__reloadThb\n";
				defShortcuts += "0::Ctrl+P::__print\n";
				defShortcuts += "0::Escape::__hide\n";
				defShortcuts += "0::Q::__close\n";
				defShortcuts += "0::Shift+O::__open\n";
				defShortcuts += "0::[M] Ctrl+Wheel Down::__zoomOut\n";
				defShortcuts += "0::[M] Ctrl+Wheel Up::__zoomIn\n";
				defShortcuts += "0::[M] Ctrl+Middle Button::__zoomReset\n";
				out << defShortcuts;
				sh.close();
			} else
				qDebug() << "ERROR: Couldn't write shortcuts file! Key navigation and shortcuts wont be available!";

			// File did not exist, that means, photo was newly installed
			update = 2;

		// If the shortcut config is already there, check for the "Version=" parameter. If it's not there/old, do some upgrade
		} else {

			if(sh.open(QIODevice::ReadWrite)) {
				QTextStream s(&sh);
				QString all = s.readAll();
				if(all.indexOf("Version=") == -1) {
					all = "Version=" + version + "\n" + all;
					if(!all.endsWith("\n"))
						all += "\n";
					update = 1;
				} else if(all.indexOf("Version=" + version) == -1) {
					QStringList allSplit = all.split("\n");
					allSplit.removeFirst();
					all = "Version=" + version + "\n" + allSplit.join("\n");
					if(all.indexOf("Version=0.7") == -1 && all.indexOf("Version=0.7.1") == -1 && all.indexOf("::[M]") == -1) {
						all += "0::[M] Ctrl+Wheel Down::__zoomOut\n";
						all += "0::[M] Ctrl+Wheel Up::__zoomIn\n";
						all += "0::[M] Ctrl+Middle Button::__zoomReset\n";
					}
					update = 1;
				}
				sh.close();
				sh.remove();
				sh.open(QIODevice::WriteOnly);
				s << all;
				sh.close();
			}

		}

		// The Window has to be initialised *after* the checks above to ensure that the shortcuts and settings exist and are updated and can be loaded
		MainWindow w;
		w.show();

		// Set the file to open that the user might have passed on
		QString file_str = QString::fromUtf8(argv[1]).trimmed();
		if(file_str != "--start-in-tray") {
			w.currentfile = file_str;
			w.thumbs->currentfile = file_str;
		} else
			file_str = "";

		if(file_str != "")
			w.open->dir = QFileInfo(file_str).absolutePath();

		// Show message if photo was updated
		if(update == 1) {
			w.m->setGeometry(w.m->menuRectShown);
			w.exif->setGeometry(w.exif->rectShown);
			QString msg = QString(QMessageBox::tr("<h1>Photo was updated</h1>Photo was succesfully updated to v%1.")).arg(version) + "<br><br>";
			msg += QMessageBox::tr("It has been a while, but here's finally a new major update:") + "<br><br>";
			msg += QMessageBox::tr("- There's only one running instance of Photo possible at time.") + "<br><br>";
			msg += QMessageBox::tr("- Photo can be controlled through command line (run 'photo --help' for more details)") + "<br><br>";
			msg += QMessageBox::tr("- Photo can make use of the system tray, and can also be started minimised to it (by running 'photo --start-in-tray')") + "<br><br>";
			msg += QMessageBox::tr("- Photo supports now very basic printing") + "<br><br>";
			msg += QMessageBox::tr("- New and experimental 'Open File' dialog - <b>experimental, use at own risk!</b>") + "<br><br>";
			msg += QMessageBox::tr("- There were many other small changes and improvements, check the changelog for a better overview") + "<br><br>";
			msg += QMessageBox::tr("<b>NOTE:</b> For any new internal function, you need to set the shortcut manually! Or alternatively you can set the set of default shortcuts in the settings (that'll remove all custom shortcuts!!)") + "<br><br>";
			msg += QMessageBox::tr("If you find a bug, or if you have a question or suggestion, tell me. I'm open to any feedback I get :-)") + "<br><br>";
			msg += QMessageBox::tr("Enjoy Photo!") + "<br><br><br>";
			msg += "<i>http://photoqt.co.cc</i>";

			QMessageBox inf;
			inf.setWindowIcon(qApp->windowIcon());
			inf.setParent(0);
			inf.setText(msg);
			inf.setWindowTitle(QMessageBox::tr("Photo was successfully updated"));
			inf.exec();
			w.m->mShown = true;
			w.exif->isShown = true;
			w.m->animateMenu(0,0,true);
			w.exif->animate(0,0,true);

		// If photo was newly installed
		} else if(update == 2) {
			w.m->setGeometry(w.m->menuRectShown);
			w.exif->setGeometry(w.exif->rectShown);
			QString msg = "<h1>Welcome to Photo v0.8</h1>Photo is a fast, simple, good looking, yet powerfull and highly configureable image viewer.<br><br>";
			msg += "Although it still is a <b>pretty young project</b>, it is already <b>fairly advanced</b>, and brings pretty much all of the functions you would expect from an image viewer: Opening files, thumbnail bar, thumbnail cache, arbitrary shortcuts, compositing, zooming, rotating, basic file handling, reading exif information, and many more...<br><br>";
			msg += "You can set a <b>shortcut</b> and/or <b>mouse action</b> for pretty much <b>any external</b> program/script/..., but also several <b>internal functions</b>. Check the default shortcuts in the settings window (default shortcut \"e\").<br><br>";
			msg += "If you like this program, why not <b>contributing</b> to it in <b>translating it into your language</b> (or a language you know well enough)? Check the Photo project at <b>www.transifex.net</b> for existing languages, and if you're <b>interested</b> just drop me an email (<b>photoQt@ymail.com</b>) or leave a comment on the <b>qt-apps.org project page</b>.<br><br>";
			msg += "Also if you find a <b>bug</b>, or if you have a <b>question or suggestion</b>, tell me. I'm open to any <b>feedback</b> I get :-)<br><br>";
			msg += "<b>Enjoy Photo!</b>";
			QMessageBox inf;
			inf.setWindowIcon(qApp->windowIcon());
			inf.setParent(0);
			inf.setText(msg);
			inf.setWindowTitle(QMessageBox::tr("Photo was successfully installed"));
			inf.exec();
			w.m->mShown = true;
			w.exif->isShown = true;
			w.m->animateMenu(0,0,true);
			w.exif->animate(0,0,true);
		}

		if(QString::fromUtf8(argv[1]).trimmed() != "--start-in-tray")
			w.view->t->start(50);

		// If photo is supposed to be started minimized into system tray -> hide it
		if(startintray)
			w.hide();

		return a.exec();

	}

}
