#include "menu.h"

Menu::Menu(QWidget *parent) : QWidget(parent) {

	this->setStyleSheet("border-radius: 8px; background-color: rgba(0, 0, 0, 210)");

	// The sensitivity of the menu
	ySensitivity = 10;
	xSensitivity = 3;

	// Per default, the menu is shown at the top
	posAtBottom = true;

	viewHeight = 0;

	// This boolean can block all the menu functions
	blockAllFunctions = false;

	/////////// All the menu items ###################
	// left side
	QStringList items;
	items << "l_::_0_::_Next_::_next_::_next";
	items << "l_::_0_::_Jump_::_jump_::_jump";
	items << "l_::_0_::_Zoom_::_zoom_::_zoom";
	items << "l_::_0_::_Rotate_::_rotate_::_rotate";
	items << "l_::_0_::_Flip_::_flip_::_flip";
	items << "l_::_1_::_File_::_file_::_file";
	items << "l_::_0_::_-_::_-_::_-";
	// right side
	items << "r_::_0_::_" + tr("General Functions") + "_::_t_::_t";
	items << "r_::_1_::_" + tr("Open File") + "_::_:/img/open.png_::_open";
	items << "r_::_1_::_" + tr("Open File (<b>experimental</b>)") + "_::_:/img/open.png_::_openOld";
	items << "r_::_1_::_" + tr("Settings") + "_::_:/img/setup.png_::_settings";
	items << "r_::_1_::_" + tr("Interrupt Thumbnail Creation") + "_::_:/img/interrupt.png_::_stopThb";
	items << "r_::_1_::_" + tr("Reload Thumbnails") + "_::_:/img/reload.png_::_reloadThb";
	items << "r_::_0_::_" + tr("Hide Thumbnails") + "_::_:/img/toggle.png_::_noThumb";
	items << "r_::_0_::_" + tr("Show Exif Widget") + "_::_:/img/exif.png_::_hideMeta";
	items << "r_::_1_::_" + tr("Print (beta)") + "_::_:/img/print.png_::_print";
	items << "r_::_1_::_" + tr("About Photo") + "_::_:/img/about.png_::_about";
	items << "r_::_1_::_" + tr("Hide (System Tray)") + "_::_:/img/quit.png_::_hide";
	items << "r_::_1_::_" + tr("Quit") + "_::_:/img/quit.png_::_close";

	// The layout holding all the labels
	QVBoxLayout *mLayL = new QVBoxLayout;
	QVBoxLayout *mLayR = new QVBoxLayout;

	// The height of the menu (lines multiplied lateron by default line height
	int lines = 0;

	// Go through all the items
	for(int i = 0; i < items.length(); ++i) {

		// Get all the data of them
		QString pos = items.at(i).split("_::_").at(0);
		int close = items.at(i).split("_::_").at(1).toInt();
		QString txt = items.at(i).split("_::_").at(2);
		QString icon = items.at(i).split("_::_").at(3);
		QString data = items.at(i).split("_::_").at(4);

		// Treat "zoom" specially
		if(data == "zoom") {

			MenuItem *item1 = new MenuItem(tr("Zoom:"),"","",0,false);
			MenuItem *item2 = new MenuItem(QString(" %1 ").arg(tr("In")),"","zoomIn",0);
			MenuItem *item25 = new MenuItem("|","","",0,false);
			MenuItem *item3 = new MenuItem(QString(" %1 ").arg(tr("Out")),"","zoomOut",0);
			MenuItem *item35 = new MenuItem("|","","",0,false);
			MenuItem *item4 = new MenuItem(QString(" %1 ").arg(tr("Reset")),"","zoomReset",0);

			QHBoxLayout *zLay1 = new QHBoxLayout;
			QHBoxLayout *zLay2 = new QHBoxLayout;

			zLay1->addStretch();
			zLay1->addWidget(item1);
			zLay1->addStretch();

			zLay2->addStretch();
			zLay2->addWidget(item2);
			zLay2->addWidget(item25);
			zLay2->addWidget(item3);
			zLay2->addWidget(item35);
			zLay2->addWidget(item4);
			zLay2->addStretch();

			QVBoxLayout *zLay = new QVBoxLayout;
			zLay->addLayout(zLay1);
			zLay->addLayout(zLay2);

			connect(item2, SIGNAL(clicked()), this, SLOT(menuClicked()));
			connect(item3, SIGNAL(clicked()), this, SLOT(menuClicked()));
			connect(item4, SIGNAL(clicked()), this, SLOT(menuClicked()));

			if(pos == "r") {
				mLayR->addSpacing(5);
				mLayR->addLayout(zLay);
			} else if(pos == "l") {
				mLayL->addSpacing(5);
				mLayL->addLayout(zLay);
			}

			lines += 2;

		// Treat "rotation" specially
		} else if(data == "rotate") {

			MenuItem *item1 = new MenuItem(tr("Rotate:"),"","",0,false);
			MenuItem *item2 = new MenuItem(tr("Left"),":/img/rotateLeft.png","rotateL",0);
			MenuItem *item25 = new MenuItem("|","","",0,false);
			MenuItem *item3 = new MenuItem(tr("Reset"),"","rotate0",0);
			MenuItem *item35 = new MenuItem("|","","",0,false);
			MenuItem *item4 = new MenuItem(QString("%1 <img src=\":/img/rotateRight.png\" height=\"13\">").arg(tr("Right")),"","rotateR",0);

			QHBoxLayout *rLay1 = new QHBoxLayout;
			QHBoxLayout *rLay2 = new QHBoxLayout;

			rLay1->addStretch();
			rLay1->addWidget(item1);
			rLay1->addStretch();

			rLay2->addStretch();
			rLay2->addWidget(item2);
			rLay2->addWidget(item25);
			rLay2->addWidget(item3);
			rLay2->addWidget(item35);
			rLay2->addWidget(item4);
			rLay2->addStretch();

			QVBoxLayout *rLay = new QVBoxLayout;
			rLay->addLayout(rLay1);
			rLay->addLayout(rLay2);

			connect(item2, SIGNAL(clicked()), this, SLOT(menuClicked()));
			connect(item3, SIGNAL(clicked()), this, SLOT(menuClicked()));
			connect(item4, SIGNAL(clicked()), this, SLOT(menuClicked()));

			if(pos == "r") {
				mLayR->addSpacing(5);
				mLayR->addLayout(rLay);
			} else if(pos == "l") {
				mLayL->addSpacing(5);
				mLayL->addLayout(rLay);
			}

			lines += 2;

		// Treat "file actions" specially
		} else if(data == "file") {

			MenuItem *item1 = new MenuItem(tr("File:"),"","",0,false);
			MenuItem *item2 = new MenuItem(tr("Rename"),"","rename",0);
			MenuItem *item25 = new MenuItem("|","","",0,false);
			MenuItem *item3 = new MenuItem(tr("Copy"),"","copy",0);
			MenuItem *item35 = new MenuItem("|","","",0,false);
			MenuItem *item4 = new MenuItem(tr("Move"),"","move",0);
			MenuItem *item45 = new MenuItem("|","","",0,false);
			MenuItem *item5 = new MenuItem(tr("Delete"),"","delete",0);

			QHBoxLayout *fLay1 = new QHBoxLayout;
			QHBoxLayout *fLay2 = new QHBoxLayout;

			fLay1->addStretch();
			fLay1->addWidget(item1);
			fLay1->addStretch();

			fLay2->addStretch();
			fLay2->addWidget(item2);
			fLay2->addWidget(item25);
			fLay2->addWidget(item3);
			fLay2->addWidget(item35);
			fLay2->addWidget(item4);
			fLay2->addWidget(item45);
			fLay2->addWidget(item5);
			fLay2->addStretch();

			QVBoxLayout *fLay = new QVBoxLayout;
			fLay->addLayout(fLay1);
			fLay->addLayout(fLay2);

			connect(item2, SIGNAL(clicked()), this, SLOT(menuClicked()));
			connect(item3, SIGNAL(clicked()), this, SLOT(menuClicked()));
			connect(item4, SIGNAL(clicked()), this, SLOT(menuClicked()));
			connect(item5, SIGNAL(clicked()), this, SLOT(menuClicked()));

			if(pos == "r") {
				mLayR->addSpacing(5);
				mLayR->addLayout(fLay);
			} else if(pos == "l") {
				mLayL->addSpacing(5);
				mLayL->addLayout(fLay);
			}

			lines += 2;

		// Treat "next" specially
		} else if(data == "next") {

			MenuItem *item0 = new MenuItem(tr("Go To"),"","",0,false);
			MenuItem *item1 = new MenuItem(QString("<img src=\":/img/previous.png\" height=\"13\"> %1").arg(tr("Previous")),"","prev",0);
			MenuItem *item15 = new MenuItem("|","","",0,false);
			MenuItem *item2 = new MenuItem(QString("%1 <img src=\":/img/next.png\" height=\"13\">").arg(tr("Next")),"","next",0);

			QHBoxLayout *npLay1 = new QHBoxLayout;
			QHBoxLayout *npLay2 = new QHBoxLayout;

			npLay1->addStretch();
			npLay1->addWidget(item0);
			npLay1->addStretch();

			npLay2->addStretch();
			npLay2->addWidget(item1);
			npLay2->addWidget(item15);
			npLay2->addWidget(item2);
			npLay2->addStretch();

			QVBoxLayout *npLay = new QVBoxLayout;
			npLay->addLayout(npLay1);
			npLay->addLayout(npLay2);

			connect(item1,SIGNAL(clicked()), this, SLOT(menuClicked()));
			connect(item2,SIGNAL(clicked()), this, SLOT(menuClicked()));

			if(pos == "r") {
				mLayR->addSpacing(5);
				mLayR->addLayout(npLay);
			} else if(pos == "l") {
				mLayL->addSpacing(5);
				mLayL->addLayout(npLay);
			}

			lines += 2;

		// Treat "jump" specifically
		} else if(data == "jump") {

			MenuItem *item0 = new MenuItem(tr("Jump To"),"","",0,false);
			MenuItem *item1 = new MenuItem(QString("<img src=\":/img/first.png\" height=\"13\"> %1").arg(tr("First")),"","gotoFirstThb",0);
			MenuItem *item15 = new MenuItem("|","","",0,false);
			MenuItem *item2 = new MenuItem(QString("%1 <img src=\":/img/last.png\" height=\"13\">").arg(tr("Last")),"","gotoLastThb",0);

			QHBoxLayout *npLay1 = new QHBoxLayout;
			QHBoxLayout *npLay2 = new QHBoxLayout;

			npLay1->addStretch();
			npLay1->addWidget(item0);
			npLay1->addStretch();

			npLay2->addStretch();
			npLay2->addWidget(item1);
			npLay2->addWidget(item15);
			npLay2->addWidget(item2);
			npLay2->addStretch();

			QVBoxLayout *npLay = new QVBoxLayout;
			npLay->addLayout(npLay1);
			npLay->addLayout(npLay2);

			connect(item1,SIGNAL(clicked()), this, SLOT(menuClicked()));
			connect(item2,SIGNAL(clicked()), this, SLOT(menuClicked()));

			if(pos == "r") {
				mLayR->addSpacing(5);
				mLayR->addLayout(npLay);
			} else if(pos == "l") {
				mLayL->addSpacing(5);
				mLayL->addLayout(npLay);
			}

			lines += 2;

		// Treat "flip" specifically
		} else if(data == "flip") {

			MenuItem *item1 = new MenuItem(tr("Flip:"),"","",0,false);
			MenuItem *item2 = new MenuItem(tr("Horizontally"),":/img/flipH.png","mirrorH",0);
			MenuItem *item25 = new MenuItem("|","","",0,false);
			MenuItem *item3 = new MenuItem(QString("%1 <img src=\":/img/flipV.png\" height=\"13\">").arg(tr("Vertically")),"","mirrorV",0);

			QHBoxLayout *fLay1 = new QHBoxLayout;
			QHBoxLayout *fLay2 = new QHBoxLayout;

			fLay1->addStretch();
			fLay1->addWidget(item1);
			fLay1->addStretch();

			fLay2->addStretch();
			fLay2->addWidget(item2);
			fLay2->addWidget(item25);
			fLay2->addWidget(item3);
			fLay2->addStretch();

			QVBoxLayout *fLay = new QVBoxLayout;
			fLay->addLayout(fLay1);
			fLay->addLayout(fLay2);

			connect(item2, SIGNAL(clicked()), this, SLOT(menuClicked()));
			connect(item3, SIGNAL(clicked()), this, SLOT(menuClicked()));

			if(pos == "r") {
				mLayR->addSpacing(5);
				mLayR->addLayout(fLay);
			} else if(pos == "l") {
				mLayL->addSpacing(5);
				mLayL->addLayout(fLay);
			}

			lines += 2;

		// Treat that as a spacer
		} else if(data == "-") {

			MenuItem *item = new MenuItem("","","",0,false);

			if(pos == "r")
				mLayR->addWidget(item);
			else
				mLayL->addWidget(item);

			++lines;

		// That's a title
		} else if(data == "t") {

			MenuItem *item = new MenuItem("<center><i>" + txt + "</i></span></center>","","",0,false);

			if(pos == "r")
				mLayR->addWidget(item);
			else if(pos == "l")
				mLayL->addWidget(item);
			++lines;

		// And all the rest
		} else {

			MenuItem *item = new MenuItem(txt,icon,data,close);

			// Keep access to the "toggle" item (change it depending on toggle status)
			if(data == "noThumb" || data == "exif")
				adjustMenuItems[data] = item;

			if(pos == "r")
				mLayR->addWidget(item);
			else if(pos == "l")
				mLayL->addWidget(item);

			connect(item, SIGNAL(clicked()), this, SLOT(menuClicked()));

			++lines;

		}

	}


	QHBoxLayout *mLay = new QHBoxLayout;
	mLay->addLayout(mLayL);
	mLay->addLayout(mLayR);

	this->setLayout(mLay);

	// This is the dimension of the menu
	QDesktopWidget desk;
	menuRectShown = QRect(desk.screenGeometry().width()-580,0,530,330);
	menuRectHidden = QRect(menuRectShown.x(),-menuRectShown.height(),menuRectShown.width(),menuRectShown.height());
	this->setGeometry(menuRectHidden);

	// The animation
	ani = new QPropertyAnimation(this,"geometry");

	// Is the menu shown?
	mShown = false;

}

// Animate the menu in/out
void Menu::animateMenu(int x, int y, bool forceClose) {

	if(!blockAllFunctions) {

		QDesktopWidget desk;

		// If the menu is requested to be opened
		if(!forceClose && x >= menuRectHidden.x()-xSensitivity && x <= menuRectHidden.x()+menuRectHidden.width()+xSensitivity && ((posAtBottom && y >= 0 && y <= ySensitivity) || (!posAtBottom && y >= desk.height()-ySensitivity-viewHeight && y <= desk.height()))) {
			ani->setDuration(600);
			if(ani->state() == QPropertyAnimation::Stopped && mShown == false) {
				qDebug() << "Opening Menu";
				mShown = true;
				ani->setStartValue(menuRectHidden);
				ani->setEndValue(menuRectShown);
				ani->setEasingCurve(QEasingCurve::InBack);
				ani->start();
			}
		}

	} else
		qDebug() << "Sorry, blocked...";


	// if the menu is requested to be closed
	if(forceClose || x < this->pos().x()-xSensitivity || x > this->pos().x()+this->width()+xSensitivity || y > this->pos().y()+this->height()) {
		ani->setDuration(500);
		if(mShown && ani->state() == QPropertyAnimation::Stopped) {
			qDebug() << "Closing Menu";
			mShown = false;
			ani->setStartValue(menuRectShown);
			ani->setEndValue(menuRectHidden);
			ani->setEasingCurve(QEasingCurve::OutBack);
			ani->start();
		}
	}

}

// If an item was clicked
void Menu::menuClicked() {

	if(!blockAllFunctions) {

		MenuItem *item = (MenuItem*) sender();
		QString txt = item->data;

		int close = item->close;

		emit itemClicked(txt,close);

	} else
		qDebug() << "Sorry, blocked...";

}

// Enabling styling of widget
void Menu::paintEvent(QPaintEvent *) {
	QStyleOption o;
	o.initFrom(this);
	QPainter p(this);
	style()->drawPrimitive(QStyle::PE_Widget, &o, &p, this);
}

Menu::~Menu() { }
