#ifndef OPEN_H
#define OPEN_H

#include <QWidget>
#include <QPainter>
#include <QStyleOption>
#include <QPropertyAnimation>
#include <QtDebug>
#include <QDir>

#include <QGraphicsView>
#include <QHBoxLayout>

#include <QTextDocument>
#include <QPixmap>
#include <QRadialGradient>
#include <QGraphicsPixmapItem>
#include <QLabel>

#include <QFileSystemModel>
#include <QTreeView>
#include <QPushButton>

#include "openpixmapitem.h"
#include "thbthread.h"

class Open : public QWidget {

	Q_OBJECT

public:
	Open(QWidget *parent = 0);
	~Open();

	// The animation used
	QPropertyAnimation *ani;

	// The QRects for the different shapes
	QRect rectHidden;
	QRect aniStart;
	QRect rectShown;

	// The currently loaded dir
	QString dir;

	// Stores if widget is shown
	bool oShown;

	// The view for the items
	QGraphicsView *view;
	QGraphicsScene scene;

	// Two labels and a button shown at the lower edge of the window
	QLabel *pos;
	QLabel *beta;
	QPushButton *close;

	// The history and current position in the history
	QStringList history;
	int currentPos;

	// The image infos
	QFileInfoList imgInfo;
	QStringList imgName;

	// The thread for getting the thumbnails
	ThumbThread *openThread;

	// All the Pixmap items
	QList<Pixmap*> allPixmaps;

	// The TreeView and its model shown in the left part of the window
	QTreeView *tree;
	QFileSystemModel *model;

public slots:
	// Adjusting the QRects for the shapes of the widget
	void resizeMe(int width, int height);

	// Load a new dir
	void loadDir(bool updateHistory=true);

	// A click on a pixmap item
	void gotClick(QString itemType, QString path);

	// For navigation (called by opennav.cpp)
	void goUp();
	void goHome();
	void goBack();
	void goForward();

	// Update a thumbnail
	void updateThumb(QImage,QString,int);

	// Opening a dir in the TreeView
	void clickOnTree(QString dir);

	// Animation of the widget
	void animate();

private:
	void paintEvent(QPaintEvent *);

signals:
	// A new dir is loaded
	void loadNewDir(QString file);

	// Signal to tell opennav that it's animating
	void animating();

	// Dis-/Enable the different items of the navigation
	void enableForward(bool enable);
	void enableBack(bool enable);
	void enableUp(bool enable);

};

#endif // OPEN_H
