#include "opennav.h"

OpenNav::OpenNav(QWidget *parent) : QWidget(parent) {

	// Style the widget
	this->setStyleSheet("background-color: rgba(0,0,0,200);");

	// The QRects for the different shapes
	rectHidden = QRect(this->x(),this->y(),this->width(),-this->height());
	rectShown = QRect(this->x(),this->y(),this->width(),this->height());

	// Initially widget is hidden
	this->setGeometry(rectHidden);
	nShown = false;

	// the animation object
	ani = new QPropertyAnimation(this,"geometry");

	// A label to go a level up
	up = new OpenLabel("up");
	up->setPixmap(QIcon::fromTheme("go-up").pixmap(rectShown.height(),rectShown.height()));

	// A label to go home
	home = new OpenLabel("home");
	home->setPixmap(QIcon::fromTheme("go-home").pixmap(rectShown.height(),rectShown.height()));

	// A label to go back
	back = new OpenLabel("back");
	back->setPixmap(QIcon::fromTheme("go-previous").pixmap(rectShown.height(),rectShown.height()));

	// A label to go forwards
	forward = new OpenLabel("forward");
	forward->setPixmap(QIcon::fromTheme("go-next").pixmap(rectShown.height(),rectShown.height()));

	// main layout
	QHBoxLayout *main = new QHBoxLayout;
	main->addStretch();
	main->addWidget(back);
	main->addWidget(up);
	main->addWidget(forward);
	main->addWidget(home);
	main->addStretch();
	this->setLayout(main);

}

// Animate widget
void OpenNav::animate() {

	if(ani->state() == QPropertyAnimation::Stopped && !nShown) {
		ani->setDuration(600);
		qDebug() << "Opening Widget";
		nShown = true;
		ani->setStartValue(rectHidden);
		ani->setEndValue(rectShown);
		ani->setEasingCurve(QEasingCurve::InBack);
		ani->start();
	} else if(ani->state() == QPropertyAnimation::Stopped && nShown) {
		ani->setDuration(300);
		qDebug() << "Closing Widget";
		nShown = false;
		ani->setStartValue(rectShown);
		ani->setEndValue(rectHidden);
		ani->setEasingCurve(QEasingCurve::OutBack);
		ani->start();
	}

}

// Adjust QRects
void OpenNav::resizeMe(int width, int height) {

	rectHidden = QRect(100,height+100,width-200,height+scrollbarheight);
	rectShown = QRect(0,0,width,height+scrollbarheight);

	up->setPixmap(QIcon::fromTheme("go-up").pixmap(rectShown.height(),rectShown.height()));
	home->setPixmap(QIcon::fromTheme("go-home").pixmap(rectShown.height(),rectShown.height()));
	back->setPixmap(QIcon::fromTheme("go-previous").pixmap(rectShown.height(),rectShown.height()));
	forward->setPixmap(QIcon::fromTheme("go-next").pixmap(rectShown.height(),rectShown.height()));

	if(nShown && ani->state() == QPropertyAnimation::Stopped)
		this->setGeometry(rectShown);

}

// Make widget styleable
void OpenNav::paintEvent(QPaintEvent *) {
	QStyleOption o;
	o.initFrom(this);
	QPainter p(this);
	style()->drawPrimitive(QStyle::PE_Widget, &o, &p, this);
}

OpenNav::~OpenNav() { }



// A custom label with enter & leave events
OpenLabel::OpenLabel(QString b) : QLabel() {

	this->setStyleSheet("background: transparent");

	button = b;

}

void OpenLabel::mouseReleaseEvent(QMouseEvent *) {

	emit clicked(button);

}

void OpenLabel::enterEvent(QEvent *) {

	if(this->isEnabled()) {

		QString t = "";
		if(button == "up")
			t = "go-up";
		else if(button == "home")
			t = "go-home";
		else if(button == "back")
			t = "go-previous";
		else if(button == "forward")
			t = "go-next";

		QPixmap old = QIcon::fromTheme(t).pixmap(this->height(),this->height());
		QPixmap comp(old.width(),old.height());
		comp.fill(Qt::transparent);
		QPainter paint(&comp);
		paint.setCompositionMode(QPainter::CompositionMode_SourceOver);
		paint.drawPixmap(0,-5,old.width(),old.height(),old);
		paint.end();

		this->setPixmap(comp);
	}

}

void OpenLabel::leaveEvent(QEvent *) {

	QString t = "";
	if(button == "up")
		t = "go-up";
	else if(button == "home")
		t = "go-home";
	else if(button == "back")
		t = "go-previous";
	else if(button == "forward")
		t = "go-next";

	QPixmap old = QIcon::fromTheme(t).pixmap(this->height(),this->height());


	this->setPixmap(old);

}


OpenLabel::~OpenLabel() { }
