#include "tabexif.h"

// Possibility to adjust exif settings
TabExif::TabExif(QWidget *parent) : QWidget(parent) {

	// The main layout and scroll area
	QVBoxLayout *centralLay = new QVBoxLayout;
	QScrollArea *centralScroll = new QScrollArea;
	QVBoxLayout *central = new QVBoxLayout(centralScroll);
	QWidget *centralScrollWidget = new QWidget(centralScroll->widget());
	centralScrollWidget->setSizePolicy(QSizePolicy::Preferred,QSizePolicy::Expanding);
	centralScrollWidget->setLayout(central);
	centralScroll->setWidget(centralScrollWidget);
	centralScrollWidget->show();
	centralScroll->setWidgetResizable(true);
	centralScroll->setVerticalScrollBarPolicy(Qt::ScrollBarAsNeeded);
	centralLay->addWidget(centralScroll);
	this->setLayout(centralLay);

	// The title labels, explaining what can be done here
	QLabel *title = new QLabel("<center><h1>" + tr("Exif Information") + "</h1></center>" + tr("Photo can display meta (exif) information for images. The widget for this information is on the left outside the screen and slides in when mouse gets close to it and/or when the set shortcut (default Ctrl+E) is triggered. On demand, the triggering by mouse movement can be disabled by checking the box below."));
	title->setWordWrap(true);
	central->addSpacing(10);
	central->addWidget(title);
	central->addSpacing(15);

	// This checkbox holds whether the exif widget also opens on mouse hovering a sensitive area
	exifShowsOnMouse = new QCheckBox(tr("Show exif widget only on shortcut"));
	QHBoxLayout *showLay = new QHBoxLayout;
	showLay->addStretch();
	showLay->addWidget(exifShowsOnMouse);
	showLay->addStretch();
	central->addLayout(showLay);
	central->addSpacing(5);

	// Line and text explaining what can be done below
	QLabel *line = new QLabel("<hr><br>" + tr("Photo can display a number of exif information (if available). Here you can set which ones are shown and which shall stay hidden."));
	line->setWordWrap(true);
	central->addWidget(line);
	central->addSpacing(20);

	// The checkboxes are aligned in a three-rowed grid layout
	QGridLayout *grid = new QGridLayout;

	allItems << tr("Show Filename");
	allItemsShort << "filename";

	allItems << tr("Show Filetype");
	allItemsShort << "filetype";

	allItems << tr("Show Filesize");
	allItemsShort << "filesize";

	allItems << tr("Show Dimensions");
	allItemsShort << "dimensions";

	allItems << tr("Show Make");
	allItemsShort << "make";

	allItems << tr("Show Model");
	allItemsShort << "model";

	allItems << tr("Show Software");
	allItemsShort << "software";

	allItems << tr("Show Time Photo was Taken");
	allItemsShort << "phototaken";

	allItems << tr("Show Exposure Time");
	allItemsShort << "exposuretime";

	allItems << tr("Show Flash");
	allItemsShort << "flash";

	allItems << tr("Show ISO");
	allItemsShort << "iso";

	allItems << tr("Show Scene Type");
	allItemsShort << "scenetype";

	allItems << tr("Show Focal Length");
	allItemsShort << "flength";

	allItems << tr("Show F-Number");
	allItemsShort << "fnumber";

	allItems << tr("Show Light Source");
	allItemsShort << "lightsource";

	allItems << tr("Show GPS Position");
	allItemsShort << "gps";

	// Setup the checkboxes for the exif info
	for(int i = 0; i < allItems.length(); ++i) {
		if(allItems.at(i) != "") {

			QCheckBox *check = new QCheckBox(allItems.at(i));
			allCheckBoxes.insert(allItemsShort.at(i),check);
			grid->addWidget(check,i%6,i/6);

		}

	}

	central->addLayout(grid);

	// Always/never/possible rotating/flipping for exif data
	QLabel *line2 = new QLabel("<hr><br>" + tr("Some cameras can detect - while taking the photo - if the camera was turned, and might store this information in the exif metadata. Now, if Photo finds this information, it can rotate the photos accordingly. Per default it's asking you if the rotation shall be done, but you can also set a fixed behaviour."));
	line2->setWordWrap(true);
	central->addWidget(line2);
	exifRotNev = new QRadioButton(tr("Never rotate images"));
	exifRotAlw = new QRadioButton(tr("Always rotate images"));
	exifRotAsk = new QRadioButton(tr("Always ask"));
	QHBoxLayout *rotLay = new QHBoxLayout;
	rotLay->addStretch();
	rotLay->addWidget(exifRotNev);
	rotLay->addWidget(exifRotAlw);
	rotLay->addWidget(exifRotAsk);
	rotLay->addStretch();
	central->addLayout(rotLay);
	central->addSpacing(20);


	QLabel *line3 = new QLabel("<hr><br>" + tr("If you're picture includes a GPS location, then a click on the location will open up the location in your default external browser. Here you can choose which online map to use (suggestions always welcome)."));
	line3->setWordWrap(true);
	central->addWidget(line3);
	central->addSpacing(5);
	exifGPSBrowser = new QListWidget;
	exifGPSBrowser->setSelectionBehavior(QListWidget::SelectRows);
	exifGPSBrowser->setSelectionMode(QListWidget::SingleSelection);
	exifGPSBrowser->setStyleSheet("background: transparent; border: none");
	exifGPSBrowser->addItem("maps.google.com");
	exifGPSBrowser->addItem("bing.com/maps");
	exifGPSBrowser->setFixedHeight(50);
	exifGPSBrowser->item(0)->setSelected(true);
	QHBoxLayout *gpsbrow = new QHBoxLayout;
	gpsbrow->addStretch();
	gpsbrow->addWidget(exifGPSBrowser);
	gpsbrow->addStretch();
	central->addLayout(gpsbrow);



	central->addStretch();

}

TabExif::~TabExif() { }
