#include "tabinterface.h"

Interface::Interface(QWidget *parent) : QWidget(parent) {

	// The main layout (a scrollarea)
	QVBoxLayout *centralLay = new QVBoxLayout;
	QScrollArea *centralScroll = new QScrollArea;
	QVBoxLayout *central = new QVBoxLayout(centralScroll);
	QWidget *centralScrollWidget = new QWidget(centralScroll->widget());
	centralScrollWidget->setSizePolicy(QSizePolicy::Preferred,QSizePolicy::Expanding);
	centralScrollWidget->setLayout(central);
	centralScroll->setWidget(centralScrollWidget);
	centralScrollWidget->show();
	centralScroll->setWidgetResizable(true);
	centralScroll->setVerticalScrollBarPolicy(Qt::ScrollBarAsNeeded);
	centralLay->addWidget(centralScroll);
	this->setLayout(centralLay);

	// The title of the tab
	QLabel *title = new QLabel("<center><h1>" + tr("Interface Settings") + "</h1></center>");
	central->addSpacing(10);
	central->addWidget(title);
	central->addSpacing(25);

	// Option to enable composite
	QLabel *compLabel = new QLabel("<b><span style=\"font-size: 12pt\">" + tr("Composite") + "</span></b> " + tr("(Change requires restart of photo)") + "<hr>" + tr("If composite is enabled, then it is used to make the background of photo real half tranparent. If it is <b>dis</b>abled (default), then photo is just making a screenshot before starting up and sets that as background.<br><b>Warning:</b> When the checkbox below is enabled, but composite is disabled/no compositing available, then the background of photo will be all black!"));
	compLabel->setWordWrap(true);
	QHBoxLayout *compLay = new QHBoxLayout;
	composite = new QCheckBox(tr("Use Composite"));
	compLay->addStretch();
	compLay->addWidget(composite);
	compLay->addStretch();
	central->addWidget(compLabel);
	central->addSpacing(5);
	central->addLayout(compLay);
	central->addSpacing(20);

	// Option to enable tray icon
	QLabel *trayLabel = new QLabel("<b><span style=\"font-size: 12pt\">" + tr("Tray Icon") + "</span></b><hr>" + tr("Photo can make use of the system tray. If this option is enabled, then Photo can be minimised to the system tray and loads up in an instant when you click on an image file.") + "<br>" + tr("It is also possible to start Photo minimised to the tray (e.g. at system startup) when called with \"--start-in-tray\"."));
	trayLabel->setWordWrap(true);
	QHBoxLayout *trayLay = new QHBoxLayout;
	trayUse = new QCheckBox(tr("Use Tray Icon"));
	trayLay->addStretch();
	trayLay->addWidget(trayUse);
	trayLay->addStretch();
	central->addWidget(trayLabel);
	central->addSpacing(5);
	central->addLayout(trayLay);
	central->addSpacing(20);


	// Option to enable a smooth transition when switching images (BETA)
	QLabel *fadeLabel = new QLabel("<b><span style=\"font-size:12pt\">" + tr("Smooth Transition") + "</span></b> " + tr("(Works but is not yet perfect)") + "<hr>" + tr("When switching between images, Photo can add a smooth transition to it. As of now, it only works for images that are not zoomed and not rotated. If an image is rotated/zoomed, then there wont be a transition, i.e. the switch happens suddenly (as \"normal\")."));
	fadeLabel->setWordWrap(true);
	QLabel *noFade = new QLabel(tr("No Transition"));
	QLabel *slowFade = new QLabel(tr("Long Transition"));
	QHBoxLayout *fadeLay = new QHBoxLayout;
	transition = new QSlider;
	transition->setMinimum(0);
	transition->setMaximum(10);
	transition->setOrientation(Qt::Horizontal);
	transition->setTickPosition(QSlider::TicksBelow);
	transition->setPageStep(1);
	fadeLay->addStretch();
	fadeLay->addWidget(noFade);
	fadeLay->addWidget(transition);
	fadeLay->addWidget(slowFade);
	fadeLay->addStretch();
	central->addWidget(fadeLabel);
	central->addSpacing(5);
	central->addLayout(fadeLay);
	central->addSpacing(20);

	// Option to loop trough folder, i.e. jump to first img when requesting next on last img
	QLabel *loopLabel = new QLabel("<b><span style=\"font-size: 12pt\">" + tr("Looping Through Folder") + "</span></b><hr>" + tr("When you load the last image in a folder and select \"Next\", Photo automatically jumps to the first image (and vice versa: if you select \"Previous\" while having the first image loaded, Photo jumps to the last image). Disabling this option makes Photo stop at the first/last image (i.e. selecting \"Next\"/\"Previous\" will have no effect in these two special cases)."));
	loopLabel->setWordWrap(true);
	loop = new QCheckBox(tr("Yep, loop through images in folder"));
	QHBoxLayout *loopLay = new QHBoxLayout;
	loopLay->addStretch();
	loopLay->addWidget(loop);
	loopLay->addStretch();
	central->addWidget(loopLabel);
	central->addSpacing(5);
	central->addLayout(loopLay);
	central->addSpacing(20);

	// Option to adjust the menu sensitivity
	QLabel *menuLabel = new QLabel("<b><span style=\"font-size: 12pt\">" + tr("Menu Sensitivity") + "</span></b><hr>" + tr("Here you can adjust the sensitivity of the menu. The menu open when your mouse cursor is inside of a certain area. A higher value increases this area and causes the menu to open easier. But if the value is too high, then it can affect the usability of Photo; the menu may then open also when it's not supposed to."));
	menuLabel->setWordWrap(true);
	QHBoxLayout *menuLay = new QHBoxLayout;
	QLabel *menuLabelLeft = new QLabel(tr("Low Sensitivity"));
	QLabel *menuLabelRight = new QLabel(tr("High Sensitivity"));
	menu = new QSlider;
	menu->setMinimum(1);
	menu->setMaximum(10);
	menu->setOrientation(Qt::Horizontal);
	menu->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Preferred);
	menu->setTickPosition(QSlider::TicksBelow);
	menu->setPageStep(1);
	menuLay->addStretch();
	menuLay->addWidget(menuLabelLeft);
	menuLay->addWidget(menu);
	menuLay->addWidget(menuLabelRight);
	menuLay->addStretch();
	central->addWidget(menuLabel);
	central->addSpacing(5);
	central->addLayout(menuLay);
	central->addSpacing(20);

	// Option to enble quitting of Photo on click on grey area (outside of image)
	QLabel *greyLabel = new QLabel("<b><span style=\"font-size: 12pt\">" + tr("Close on Click in empty area") + "</span></b><hr>" + tr("This option makes Photo a bit like the javascript image viewers you find on many websites. A click outside of the image on the grey background will close the app.<br>It can be a nice feature, Photo will feel even more like a \"floating layer\". However, you might at times close Photo accidentally, what might be a bit annoying.") + "<br><br>" + tr("<b>Note:</b>: If you set a mouseclick as shortcut, then this option wont have any effect!"));
	greyLabel->setWordWrap(true);
	QHBoxLayout *greyLay = new QHBoxLayout;
	grey = new QCheckBox(tr("Close on Click in empty area"));
	greyLay->addStretch();
	greyLay->addWidget(grey);
	greyLay->addStretch();
	central->addWidget(greyLabel);
	central->addSpacing(5);
	central->addLayout(greyLay);
	central->addSpacing(20);

	// Option to hide the label displayed in mainwindow
	QLabel *hideLabel = new QLabel("<b><span style=\"font-size: 12pt\">" + tr("Hide Text Labels") + "</span></b><hr>" + tr("Here you can hide the text labels shown in the main area: The Counter in the top left corner, the filename following the counter, and the \"X\" displayed in the top right corner.") + "<br><br>" + tr("<b>Hint:</b> The labels can also be easily hidden by simply rightclicking on them and selecting \"Hide\"."));
	hideLabel->setWordWrap(true);
	QHBoxLayout *hideLay = new QHBoxLayout;
	QVBoxLayout *checkLay = new QVBoxLayout;
	hideCounter = new QCheckBox(tr("Hide Counter"));
	hideFilename = new QCheckBox(tr("Hide Filename"));
	hideX = new QCheckBox(tr("Hide \"X\" (Closing)"));
	checkLay->addWidget(hideCounter);
	checkLay->addWidget(hideFilename);
	checkLay->addWidget(hideX);
	hideLay->addStretch();
	hideLay->addLayout(checkLay);
	hideLay->addStretch();
	central->addWidget(hideLabel);
	central->addSpacing(5);
	central->addLayout(hideLay);
	central->addSpacing(20);

	central->addStretch();

}



Interface::~Interface() { }
