#include "tabshortcuts.h"

Shortcuts::Shortcuts(QWidget *parent) : QWidget(parent) {

	// This variable holds the row number marked as "hovered"
	hovRow = -1;

	// The key-window for detecting key combinations
	key = new Key;

	// These two stringlists hold info about the internal functions
	allIntern << "__openOld";
	allInternRead << tr("Open new file");

	allIntern << "__next";
	allInternRead << tr("Next Image");

	allIntern << "__prev";
	allInternRead << tr("Previous Image");

	allIntern << "__zoomIn";
	allInternRead << tr("Zoom In");

	allIntern << "__zoomOut";
	allInternRead << tr("Zoom Out");

	allIntern << "__zoomReset";
	allInternRead << tr("Reset Zoom");

	allIntern << "__zoomActual";
	allInternRead << tr("Zoom to actual size");

	allIntern << "__settings";
	allInternRead << tr("Show settings");

	allIntern << "__rotateR";
	allInternRead << tr("Rotate Right");

	allIntern << "__rotateL";
	allInternRead << tr("Rotate Left");

	allIntern << "__rotate0";
	allInternRead << tr("Reset Rotation");

	allIntern << "__flipH";
	allInternRead << tr("Flip Horizontally");

	allIntern << "__flipV";
	allInternRead << tr("Flip Vertically");

	allIntern << "__rename";
	allInternRead << tr("Rename File");

	allIntern << "__delete";
	allInternRead << tr("Delete File");

	allIntern << "__copy";
	allInternRead << tr("Copy file to a new location");

	allIntern << "__move";
	allInternRead << tr("Move file to a new location");

	allIntern << "__about";
	allInternRead << tr("About photo");

	allIntern << "__noThumb";
	allInternRead << tr("Hide/Show Thumbnails");

	allIntern << "__stopThb";
	allInternRead << tr("Interrupt Thumbnail creation");

	allIntern << "__reloadThb";
	allInternRead << tr("Reload Thumbnails");

	allIntern << "__close";
	allInternRead << tr("Quit photo");

	allIntern << "__hide";
	allInternRead << tr("Hide to System Tray");

	allIntern << "__hideMeta";
	allInternRead << tr("Hide/Show Exif Info");

	allIntern << "__gotoFirstThb";
	allInternRead << tr("Go to first Image");

	allIntern << "__gotoLastThb";
	allInternRead << tr("Go to last Image");

	allIntern << "__print";
	allInternRead << tr("Print Image");

	allIntern << "__open";
	allInternRead << tr("Open new file (experimental)");


	// The main layout of this tab
	shview = new QHBoxLayout;
	this->setLayout(shview);

	// Setup the tableview and model
	setupTableView();

	// Load all the existing shortcuts
	loadShortcuts();

	// Setup the editor area on the right
	setupWidget();

	// Select and load the first shortcut
	thisview->selectRow(0);
	loadRow(model->index(0,0),model->index(0,0));

	// A new key combination is passed on to the tab widget
	connect(key, SIGNAL(gotKeys(QString)), this, SLOT(gotNewKeys(QString)));

}

// Sets up the table with the shortcuts
void Shortcuts::setupTableView() {

	// The class "Table" is a subclass of QTableView providing an additional mousemoveevent function and a signal
	thisview = new Table;//	set->show();


	// We don't need the vertical headers
	thisview->verticalHeader()->hide();

	// The vertical header has plain informational purpose
	thisview->horizontalHeader()->setClickable(false);

	// Some styling optionseditor
	thisview->setShowGrid(false);
	thisview->setAlternatingRowColors(true);
	thisview->setStyleSheet("background: transparent");

	// Exactly one row can be selected at a time
	thisview->setSelectionMode(QTableView::SingleSelection);
	thisview->setSelectionBehavior(QTableView::SelectRows);

	// Editing is not possible
	thisview->setEditTriggers(QTableView::NoEditTriggers);

	// The mouse-tracking is needed for the hover effect
	thisview->setMouseTracking(true);

	// We make use of Qt's model/view system
	model = new QStandardItemModel;

	// The model has a total of 3 columns
	model->setColumnCount(3);

	thisview->setModel(model);

	// The tableview is packed in a VBoxLayout, cause we add two buttons below it for adding/removing shortcuts
	QVBoxLayout *tableLay = new QVBoxLayout;
	tableLay->addWidget(thisview);

	// This is the layout of the total tab, containing the table (and more)
	shview->addLayout(tableLay);

	// Two buttons below the table for adding/removing shortcuts
	QHBoxLayout *shButtons = new QHBoxLayout;
	QPushButton *add = new QPushButton(tr("Add Shortcut (Ctrl++)"));
	add->setShortcut(QKeySequence("Ctrl++"));
	QPushButton *rem = new QPushButton(tr("Remove Shortcut (Ctrl+-)"));
	rem->setShortcut(QKeySequence("Ctrl+-"));
	shButtons->addWidget(add);
	shButtons->addWidget(rem);
	shButtons->addStretch();
	tableLay->addLayout(shButtons);

	// Adding a row at the end and setting the focus into it
	connect(add, SIGNAL(clicked()), this, SLOT(addNewShortcut()));
	connect(add, SIGNAL(clicked()), this, SLOT(focusLastRow()));

	// Removing the currently selected row
	connect(rem, SIGNAL(clicked()), this, SLOT(removeShortcut()));

	// Mark the whole row when hovering an item
	connect(thisview, SIGNAL(hoverPos(QPoint)),this, SLOT(hoverRow(QPoint)));

	// Load a shortcut into editor window at the right on click
	connect(thisview->selectionModel(), SIGNAL(currentRowChanged(QModelIndex,QModelIndex)), this, SLOT(loadRow(QModelIndex,QModelIndex)));

}

// Setting up the editor widget at the right (for editing shortcuts)
void Shortcuts::setupWidget() {

	// Some layout stuff
	editor = new QWidget(this);
	editor->setFixedWidth(250);
	editor->setSizePolicy(QSizePolicy::Preferred,QSizePolicy::Expanding);
	shview->addWidget(editor);

	// A title on top of the widget
	QLabel *eT = new QLabel("<center><h1>" + tr("Shortcut") + "</h1></center>");
	eT->setWordWrap(true);
	QVBoxLayout *eLay = new QVBoxLayout;
	eLay->addWidget(eT);
	eLay->addSpacing(20);

	// The radio button for setting a key combo
	shSh = new QRadioButton(tr("Key Combination"));
	shSh->setChecked(true);
	eLay->addWidget(shSh);
	connect(shSh, SIGNAL(clicked()), this, SLOT(updateData()));

	// The button for the key shortcut button (and label)
	QLabel *shEditLabel = new QLabel("<b>" + tr("Click to change: (Ctrl+B)") + "</b>");
	shEditLabel->setWordWrap(true);
	eLay->addWidget(shEditLabel);
	shEdit = new QPushButton;
	shEdit->setFixedWidth(editor->width()/2.0);
	QHBoxLayout *sheditLay = new QHBoxLayout;
	sheditLay->addStretch();
	sheditLay->addWidget(shEdit);
	sheditLay->addStretch();
	eLay->addLayout(sheditLay);

	// The radio button for setting a mouse action
	shMou = new QRadioButton(tr("Mouse Action"));
	eLay->addWidget(shMou);

	// We put them into a group, because they should work for themselves and not affect other radio buttons in the widget as well
	QButtonGroup *shBut = new QButtonGroup;
	shBut->addButton(shSh);
	shBut->addButton(shMou);

	connect(shSh, SIGNAL(toggled(bool)), shEditLabel, SLOT(setEnabled(bool)));
	connect(shSh, SIGNAL(toggled(bool)), shEdit, SLOT(setEnabled(bool)));
	connect(shSh, SIGNAL(clicked(bool)), this, SLOT(gotMouseCombo()));

	// The modifiers (for mouse action)
	mod = new QComboBox;
	mod->addItem("---");
	mod->addItem("Ctrl");
	mod->addItem("Alt");
	mod->addItem("Shift");
	mod->addItem("Ctrl+Alt");
	mod->addItem("Ctrl+Shift");
	mod->addItem("Alt+Shift");
	mod->addItem("Ctrl+Alt+Shift");

	// The mouse buttons (for mouse action)
	mouse = new QComboBox;
	mouse->addItem("Left Button");
	mouse->addItem("Right Button");
	mouse->addItem("Middle Button");
	mouse->addItem("Wheel Up");
	mouse->addItem("Wheel Down");

	connect(mod, SIGNAL(activated(int)), this, SLOT(gotMouseCombo()));
	connect(mouse, SIGNAL(activated(int)), this, SLOT(gotMouseCombo()));

	// per default a key combo is set, i.e. these widgets are disabled
	mod->setEnabled(false);
	mouse->setEnabled(false);

	connect(shMou, SIGNAL(toggled(bool)), mod, SLOT(setEnabled(bool)));
	connect(shMou, SIGNAL(toggled(bool)), mouse, SLOT(setEnabled(bool)));
	connect(shMou, SIGNAL(clicked()), this, SLOT(updateData()));

	// The layout for setting mouse actions
	QHBoxLayout *modMouseLay = new QHBoxLayout;
	modMouseLay->addStretch();
	modMouseLay->addWidget(mod);
	modMouseLay->addWidget(mouse);
	modMouseLay->addStretch();
	eLay->addLayout(modMouseLay);
	eLay->addSpacing(20);

	// The command can be either internal or external
	QLabel *exeEditLabel = new QLabel("<b>" + tr("Current command:") + "</b>");
	exeEditLabel->setWordWrap(true);
	exeInternButton = new QRadioButton(tr("&Internal commands"));
	exeInternButton->setChecked(true);
	exeIntern = new QListWidget;
	exeExternButton = new QRadioButton(tr("&External command"));
	exeExtern = new QLineEdit;
	QHBoxLayout *exeExtLay = new QHBoxLayout;
	exeExtBrowse = new QPushButton("...");
	exeExtLay->addWidget(exeExtern);
	exeExtLay->addWidget(exeExtBrowse);
	eLay->addWidget(exeEditLabel);
	eLay->addSpacing(5);
	eLay->addWidget(exeInternButton);
	eLay->addWidget(exeIntern);
	eLay->addSpacing(5);
	eLay->addWidget(exeExternButton);
	eLay->addLayout(exeExtLay);
	eLay->addSpacing(10);
	connect(exeExtBrowse, SIGNAL(clicked()), this, SLOT(getExeExtFile()));

	// A short text informing about the two possible placeholders
	exeExternPlaceholder = new QLabel(tr("You can use the following placeholders for external commands:<br>%f = current file<br>%d = current directory"));
	exeExternPlaceholder->setWordWrap(true);
	eLay->addWidget(exeExternPlaceholder);
	eLay->addSpacing(20);

	// Inserting all internal commans into the list
	for(int i = 0; i < allIntern.length(); ++i) {
		exeIntern->addItem(allInternRead.at(i));
		exeIntern->item(i)->setData(Qt::UserRole+1,allIntern.at(i));
	}

	// Checkbox, quitting photo after executing above command (only at external commands)
	clEdit = new QCheckBox(tr("&Quit after executing command?"));
	eLay->addWidget(clEdit);
	eLay->addStretch();

	// A button that replaces the current set of shortcuts with a default one
	QPushButton *restoreDefault = new QPushButton(tr("Restore default shortcuts (Ctrl+R)"));
	restoreDefault->setShortcut(QKeySequence("Ctrl+R"));
	QHBoxLayout *restoreLay = new QHBoxLayout;
	restoreLay->addStretch();
	restoreLay->addWidget(restoreDefault);
	eLay->addLayout(restoreLay);

	// Set this layout to the editor widget
	editor->setLayout(eLay);

	// The radio buttons en-/disable each other's lineedit
	connect(exeInternButton, SIGNAL(toggled(bool)), exeIntern, SLOT(setEnabled(bool)));
	connect(exeExternButton, SIGNAL(toggled(bool)), exeExtern, SLOT(setEnabled(bool)));
	connect(exeExternButton, SIGNAL(toggled(bool)), exeExtBrowse, SLOT(setEnabled(bool)));
	connect(exeExternButton, SIGNAL(toggled(bool)), clEdit, SLOT(setEnabled(bool)));
	connect(exeExternButton, SIGNAL(toggled(bool)), exeExternPlaceholder, SLOT(setEnabled(bool)));

	// A click on the button with the shortcut opens a small widget for detecting a new key combination
	connect(shEdit, SIGNAL(clicked()), this, SLOT(getNewKeys()));
	QShortcut *edSh = new QShortcut(QKeySequence("Ctrl+B"),editor);
	connect(edSh, SIGNAL(activated()), this, SLOT(getNewKeys()));

	// The table is kept in sync with changes in the editor widget
	connect(exeIntern, SIGNAL(activated(QModelIndex)), this, SLOT(updateData()));
	connect(exeInternButton, SIGNAL(clicked()),this, SLOT(updateData()));
	connect(exeExtern, SIGNAL(textEdited(QString)), this, SLOT(updateData()));
	connect(exeExternButton, SIGNAL(clicked()), this, SLOT(updateData()));
	connect(clEdit, SIGNAL(clicked()), this, SLOT(updateData()));

	// Restoring the default shortcut set
	connect(restoreDefault, SIGNAL(clicked()), this, SLOT(restoreDefaultShortcuts()));

}

// This function asks the user for a file and sets it as executable for currently selected row
void Shortcuts::getExeExtFile() {

	QString dir = QDir::homePath();
	if(exeExtern->text().trimmed().length())
		dir = QFileInfo(exeExtern->text()).absolutePath();

	QString file = QFileDialog::getOpenFileName(this,tr("Select executable"),dir);

	// We only use it, if actually a file was selected (cancel = empty string)
	if(file != "") {
		exeExtern->setText(file);
		updateData();
	}

}

// Loading all the current shortcuts (from file)
void Shortcuts::loadShortcuts() {

	// This list holds a boolean for each row, storing if Photo should be quit on executing shortcut or not
	toClose.clear();

	qDebug() << "Setup: Loading all shortcuts";

	QFile file(QDir::homePath() + "/.photo/shortcuts");

	if(file.open(QIODevice::ReadOnly)) {

		// Clear the model and set column count to 3 again (needed for reloading the shortcuts)
		model->clear();
		model->setColumnCount(3);

		// Go through the file line by line
		QTextStream in(&file);
		QStringList sh = in.readAll().split("\n");

		// The integer j counts the actual shortcuts (the int i is counting the lines -> not reliable)
		int j = 0;
		for(int i = 0; i < sh.length(); ++i) {

			// If the line is a valid shortcut
			if(sh.at(i).trimmed() != "" && sh.at(i).contains("::")) {

				// Add a new row to the table
				addNewShortcut();

				// 3 strings: close, shortcut, command
				QString c = sh.at(i).split("::").at(0);
				if(c.toInt()) {
					c = tr("Quit");
					toClose.replace(j,true);
				} else {
					c = tr("Don't quit");
					toClose.replace(j,false);
				}

				QString shortcut = sh.at(i).split("::").at(1);

				// In the table, all internal function get a "::" prepended to make is easily visible that they are internal functions
				QString todo = "";
				int readable = allIntern.indexOf(sh.at(i).split("::").at(2));
				if(readable != -1 && readable < allInternRead.length())
					todo = ":: " + allInternRead.at(readable);
				else
					todo = sh.at(i).split("::").at(2);

				// Set up the items
				QStandardItem *one = new QStandardItem(shortcut);
				one->setData(Qt::AlignCenter, Qt::TextAlignmentRole);
				one->setToolTip(shortcut);
				QStandardItem *two = new QStandardItem(todo);
				QStandardItem *three = new QStandardItem(c);
				three->setData(Qt::AlignCenter, Qt::TextAlignmentRole);

				// The "and close" option doesn't apply for internal functions
				if(todo.startsWith("::")) {
					three->setEnabled(false);
					three->setText(tr("N/A"));
				}

				// Set items to table
				model->setItem(model->rowCount()-1,0,one);
				model->setItem(model->rowCount()-1,1,two);
				model->setItem(model->rowCount()-1,2,three);

				++j;

			}

		}

	// File couldn't be read
	} else
		qDebug() << "Couldn't read existing shortcuts...";

}

// When the widget is resized, the column width needs to be adjusted
void Shortcuts::resizeEvent(QResizeEvent *) {

	// The resize code ismoved to a normal function, cause it is also called at other places programmatically
	doresizing();

}

// The actualy column resizing
void Shortcuts::doresizing() {

	thisview->setColumnWidth(0,thisview->width()*0.30);
	thisview->setColumnWidth(1,thisview->width()*0.50);
	thisview->setColumnWidth(2,thisview->width()*0.01);
	thisview->horizontalHeader()->setStretchLastSection(true);

	// The width is fixed
	thisview->horizontalHeader()->setResizeMode(0,QHeaderView::Fixed);
	thisview->horizontalHeader()->setResizeMode(1,QHeaderView::Fixed);
	thisview->horizontalHeader()->setResizeMode(2,QHeaderView::Fixed);

	// Set the text on the header items
	model->setHeaderData(0, Qt::Horizontal, tr("Shortcut"), Qt::DisplayRole);
	model->setHeaderData(1, Qt::Horizontal, tr("Execute"), Qt::DisplayRole);
	model->setHeaderData(2, Qt::Horizontal, tr("Quit?"), Qt::DisplayRole);

}

// Add a new row to the table
void Shortcuts::addNewShortcut() {

	model->setRowCount(model->rowCount()+1);
	toClose.append(false);

	QStandardItem *one = new QStandardItem(tr("[Detect]"));
	one->setData(Qt::AlignCenter, Qt::TextAlignmentRole);
	QStandardItem *two = new QStandardItem(tr("[Execute]"));
	QStandardItem *three = new QStandardItem(tr("Don't quit"));
	three->setData(Qt::AlignCenter, Qt::TextAlignmentRole);

	model->setItem(model->rowCount()-1,0,one);
	model->setItem(model->rowCount()-1,1,two);
	model->setItem(model->rowCount()-1,2,three);

}

// Set the focus to the last row and ensure it's visible
void Shortcuts::focusLastRow() {

	if(model->rowCount() > 0) {
		thisview->selectionModel()->clearSelection();
		thisview->selectionModel()->select(model->index(model->rowCount()-1,0), QItemSelectionModel::Select);
		thisview->selectionModel()->select(model->index(model->rowCount()-1,1), QItemSelectionModel::Select);
		thisview->selectionModel()->select(model->index(model->rowCount()-1,2), QItemSelectionModel::Select);
		loadRow(model->index(model->rowCount()-1,0),model->index(model->rowCount()-1,0));
		thisview->scrollToBottom();
	}

}

// A hover effect
void Shortcuts::hoverRow(QPoint p) {

	// A brush with background color
	QBrush b(Qt::SolidPattern);
	b.setColor(qRgba(85, 181, 235,200));

	int row = thisview->indexAt(p).row();

	// Reset the previous hovered row
	if(hovRow != -1) {
		model->item(hovRow,0)->setBackground(QBrush());
		model->item(hovRow,1)->setBackground(QBrush());
		model->item(hovRow,2)->setBackground(QBrush());
	}
	// Mark the currently hovered row
	if(row != -1) {
		model->item(row,0)->setBackground(b);
		model->item(row,1)->setBackground(b);
		model->item(row,2)->setBackground(b);
	}

	hovRow = row;

}

// Load a new row
void Shortcuts::loadRow(QModelIndex index,QModelIndex) {

	if(index.row() != -1) {

		// We take the shortcut and store it in a string for further analysing
		QString sh = model->item(index.row(),0)->text();

		// If shortcut is a mouse action
		if(sh.startsWith("[M]")) {

			// Check the radio button
			shMou->setChecked(true);

			// First load the modifier
			if(sh.contains("Ctrl+Alt+Shift"))
				mod->setCurrentIndex(mod->findText("Ctrl+Alt+Shift"));
			else if(sh.contains("Alt+Shift"))
				mod->setCurrentIndex(mod->findText("Alt+Shift"));
			else if(sh.contains("Ctrl+Shift"))
				mod->setCurrentIndex(mod->findText("Ctrl+Shift"));
			else if(sh.contains("Ctrl+Alt"))
				mod->setCurrentIndex(mod->findText("Ctrl+Alt"));
			else if(sh.contains("Shift"))
				mod->setCurrentIndex(mod->findText("Shift"));
			else if(sh.contains("Alt"))
				mod->setCurrentIndex(mod->findText("Alt"));
			else if(sh.contains("Ctrl"))
				mod->setCurrentIndex(mod->findText("Ctrl"));
			else
				mod->setCurrentIndex(mod->findText("---"));

			// Then also load the mouse button
			if(sh.contains("Left Button"))
				mouse->setCurrentIndex(mouse->findText("Left Button"));
			else if(sh.contains("Right Button"))
				mouse->setCurrentIndex(mouse->findText("Right Button"));
			else if(sh.contains("Middle Button"))
				mouse->setCurrentIndex(mouse->findText("Middle Button"));
			else if(sh.contains("Wheel Up"))
				mouse->setCurrentIndex(mouse->findText("Wheel Up"));
			else if(sh.contains("Wheel Down"))
				mouse->setCurrentIndex(mouse->findText("Wheel Down"));


		// Otherwise it's a key combo
		} else {
			shSh->setChecked(true);
			shEdit->setText(sh);
		}

		// Get the execute command
		QString todocommand = model->item(index.row(),1)->text().remove(0,3);

		// If it's an external command
		if(allInternRead.indexOf(todocommand) == -1) {
			exeExternButton->setChecked(true);
			exeIntern->item(0)->setSelected(true);
			exeIntern->scrollToItem(exeIntern->item(0));
			exeExtern->setText(model->item(index.row(),1)->text());
		// If it's an internal command
		} else {
			exeIntern->item(allInternRead.indexOf(todocommand))->setSelected(true);
			exeIntern->scrollToItem(exeIntern->item(allInternRead.indexOf(todocommand)));
			exeExtern->setText("");
			exeInternButton->setChecked(true);
		}

		// Get the "and quit" status
		if(toClose.length() < index.row())
			clEdit->setChecked(toClose.at(index.row()));
		else
			clEdit->setChecked(false);

		// Set the checkboxes/lineedits/... depending on internal or external function
		DisEnableCommands(index.row());

		// Check if mouse combo is unique (boolean false = just check for buttons on right, don't mess with table -> otherwise crash at startup!)
		if(model->rowCount() > 0)
			gotMouseCombo(false);

	}

}

// Update elements depending on internal/external function
void Shortcuts::DisEnableCommands(int row) {

	if(exeInternButton->isChecked()) {
		exeIntern->setEnabled(true);
		exeExtern->setEnabled(false);
		exeExtBrowse->setEnabled(false);
		clEdit->setEnabled(false);
		exeExternPlaceholder->setEnabled(false);
		if(row > -1)
			model->item(row,2)->setEnabled(false);
	} else {
		exeIntern->setEnabled(false);
		exeExtern->setEnabled(true);
		exeExtBrowse->setEnabled(true);
		clEdit->setEnabled(true);
		exeExternPlaceholder->setEnabled(true);
		if(row > -1)
			model->item(row,2)->setEnabled(true);
	}


}

// Syncs the editor data with the table data
void Shortcuts::updateData() {

	// Get the currently selected (and thus loaded) row
	int row = thisview->selectionModel()->selectedIndexes().at(0).row();

	// Sync the shortcut
	if(!model->item(row,0)->text().startsWith("[M]") && shMou->isChecked())
		gotMouseCombo();
	else if(shSh->isChecked())
		// Set the (possibly new) shortcut
		model->item(row,0)->setText(shEdit->text());

	// Set the command and quit status
	if(exeInternButton->isChecked()) {
		model->item(row,1)->setText(":: " + exeIntern->selectedItems().at(0)->text());
		model->item(row,2)->setText(tr("N/A"));
		model->item(row,2)->setEnabled(false);
	} else {
		model->item(row,1)->setText(exeExtern->text());
		model->item(row,2)->setEnabled(true);
		if(clEdit->isChecked()) {
			model->item(row,2)->setText(tr("Quit"));
			toClose.replace(row,true);
		} else {
			model->item(row,2)->setText(tr("Don't quit"));
			toClose.replace(row,false);
		}
	}

}

// Save shortcuts to file
void Shortcuts::saveData() {

	QString conf = "Version=" + version + "\n";

	for(int i = 0; i < model->rowCount(); ++i) {

		// Get the "and close" status
		QString close = "0";
		if(toClose.at(i))
			close = "1";

		// Get the execute command
		QString exe = model->item(i,1)->text();
		if(exe.startsWith("::")) {
			close = "0";
			exe = exe.remove(0,3);
			int index = allInternRead.indexOf(exe);
			if(index != -1 && index < allIntern.length())
				exe = allIntern.at(index);
			else
				exe = "?";
		}

		// Add data to file content string
		conf += close + "::" + model->item(i,0)->text() + "::" + exe + "\n";


	}

	// Create and write config file for shortcuts
	QFile file(QDir::homePath() + "/.photo/shortcuts");
	file.remove();
	if(file.open(QIODevice::ReadWrite)) {
		qDebug() << "Saving shortcuts";
		QTextStream out(&file);
		out << conf;
		file.close();
	} else
		qDebug() << "Couldn't save shortcuts";

	emit updateShortcuts();

}

// Remove the currently selected shortcut
void Shortcuts::removeShortcut() {

	// Ask for confirmation
	int msg = QMessageBox::information(this,tr("Delete Shortcut?"),tr("Do you really want to delete this shortcut? This step cannot be reversed."),QMessageBox::Yes | QMessageBox::Default, QMessageBox::No);

	if(msg == QMessageBox::Yes) {

		// Remove row and (if none selected) select the last row
		int row = thisview->selectionModel()->selectedIndexes().at(0).row();
		model->removeRow(row);
		if(thisview->selectionModel()->selectedIndexes().length() == 0) {
			thisview->selectRow(model->rowCount()-1);
		}

	}

}

// Open the "key" widget for detecting a new key combination
void Shortcuts::getNewKeys() {

	if(shSh->isChecked()) {

		qDebug() << "Setup: Detecting new shortcut";

		key->det->setText(tr("Press Keys"));
		this->setEnabled(false);
		key->det->setFocus();

		QRect r = key->frameGeometry();
		QDesktopWidget desk;
		r.moveCenter(desk.availableGeometry().center());
		key->move(r.topLeft());

		key->exec();
	}

}

// Handling the key combination submitted by "key" widget
void Shortcuts::gotNewKeys(QString k) {

	this->activateWindow();

	if(k == "") {

		qDebug() << "No new key combination detected.";
		this->setEnabled(true);

	} else {

		// Check if shortcut is either new or unchanged
		int there = 0;
		for(int i = 0; i < model->rowCount(); ++i) {
			if(model->item(i,0)->text() == k.trimmed() && i != thisview->selectionModel()->selectedIndexes().at(0).row())
				there = 1;
		}

		if(there == 1) {
			QMessageBox::critical(0,tr("Shortcut exists already"),QString(tr("Sorry, the shortcut '%1' is already occupied. Please choose another one.")).arg(k));
			key->everythingsokay = 0;
		} else {
			qDebug() << "Got new key combination";
			this->setEnabled(true);
			shEdit->setText(k);
			model->item(thisview->selectionModel()->selectedIndexes().at(0).row(),0)->setText(k);
			model->item(thisview->selectionModel()->selectedIndexes().at(0).row(),0)->setToolTip(k);
		}

	}

}

void Shortcuts::gotMouseCombo(bool andSetToTable) {

	// Get the new mouse action
	QString k = "";
	if(mod->currentText() != "---")
		k = "[M] " + mod->currentText() + "+" + mouse->currentText();
	else
		k = "[M] " + mouse->currentText();

	// Except when andSetToTable==false (e.g. at startup call), sync it to listview
	if(andSetToTable && shMou->isChecked()) {
		qDebug() << "Got new Key/Mouse combination";
		model->item(thisview->selectionModel()->selectedIndexes().at(0).row(),0)->setText(k);
		model->item(thisview->selectionModel()->selectedIndexes().at(0).row(),0)->setToolTip(k);
	}

	// Check, if the mouse action exists already somewhere else
	int there = 0;
	for(int i = 0; i < model->rowCount(); ++i) {
		if(model->item(i,0)->text() == k.trimmed())
			there += 1;
	}

	// If it exists twice (current row and somewhere else) and if shMou is checked -> red background
	if(there == 2 && shMou->isChecked()) {
		mod->setStyleSheet("background: rgba(255,0,0,190)");
		mouse->setStyleSheet("background: rgba(255,0,0,190)");
	// If all is fine -> normal background
	} else {
		mod->setStyleSheet("");
		mouse->setStyleSheet("");
	}

}

// Deletes all current shortcuts and enters the default shortcut set
void Shortcuts::restoreDefaultShortcuts() {

	int msg = QMessageBox::warning(0,tr("Restore default shortcuts"),tr("<b>Warning!</b><br><br>This step <b>deletes</b> all you existing shortcuts and replaces them with the default set of shortcuts! This step cannot be reversed! Continue?"),QMessageBox::Yes | QMessageBox::Default, QMessageBox::No);

	if(msg == QMessageBox::Yes) {

		QFile file(QDir::homePath() + "/.photo/shortcuts");
		if(!file.open(QIODevice::ReadWrite))
			QMessageBox::critical(0,tr("Couldn't open file"),tr("<b>ERROR!</b><br>I wasn't able to open the file that stores your shortcuts. Please ensure, that you have both <b>read and write access</b> to your home directory!"));
		else {
			file.close();
			file.remove();
			file.open(QIODevice::ReadWrite);
			QTextStream out(&file);
			QString defShortcuts = "";
			defShortcuts += "Version=" + version + "\n";
			defShortcuts += "0::O::__openOld\n";
			defShortcuts += "0::Right::__next\n";
			defShortcuts += "0::Space::__next\n";
			defShortcuts += "0::Left::__prev\n";
			defShortcuts += "0::Backspace::__prev\n";
			defShortcuts += "0::+::__zoomIn\n";
			defShortcuts += "0::Ctrl++::__zoomIn\n";
			defShortcuts += "0::-::__zoomOut\n";
			defShortcuts += "0::Ctrl+-::__zoomOut\n";
			defShortcuts += "0::0::__zoomReset\n";
			defShortcuts += "0::Ctrl+1::__zoomActual\n";
			defShortcuts += "0::R::__rotateR\n";
			defShortcuts += "0::L::__rotateL\n";
			defShortcuts += "0::Ctrl+0::__rotate0\n";
			defShortcuts += "0::Ctrl+H::__flipH\n";
			defShortcuts += "0::Ctrl+V::__flipV\n";
			defShortcuts += "0::Ctrl+E::__hideMeta\n";
			defShortcuts += "0::E::__settings\n";
			defShortcuts += "0::F2::__rename\n";
			defShortcuts += "0::Ctrl+C::__copy\n";
			defShortcuts += "0::Ctrl+M::__move\n";
			defShortcuts += "0::Delete::__delete\n";
			defShortcuts += "0::I::__about\n";
			defShortcuts += "0::H::__noThumb\n";
			defShortcuts += "0::S::__stopThb\n";
			defShortcuts += "0::Ctrl+R::__reloadThb\n";
			defShortcuts += "0::Ctrl+P::__print\n";
			defShortcuts += "0::Escape::__hide\n";
			defShortcuts += "0::Q::__close\n";
			defShortcuts += "0::Shift+O::__open\n";
			defShortcuts += "0::[M] Ctrl+Wheel Down::__zoomOut\n";
			defShortcuts += "0::[M] Ctrl+Wheel Up::__zoomIn\n";
			defShortcuts += "0::[M] Ctrl+Middle Button::__zoomReset\n";
			out << defShortcuts;
			file.close();
		}

		// Reload table list
		loadShortcuts();
		doresizing();
		thisview->selectRow(0);

	}

}

// When settings window is closed, ensure, that the mainwindow is focussed again
void Shortcuts::closeEvent(QCloseEvent *) {
	emit focusParentWindow();
}

Shortcuts::~Shortcuts() {
}




/****************************************
  #######################################
  #######################################

  THIS CLASS ADDS ADDITIONAL FUNCTIONALITY
	    TO THE QTABLEVIEW

  #######################################
  #######################################
  ***************************************/

// Function to recognise and handle the row hover effect
void Table::mouseMoveEvent(QMouseEvent *e) {
	emit hoverPos(e->pos());
}
bool Table::event(QEvent *e) {
	if(e->type() == QEvent::Leave)
		emit hoverPos(QPoint(-1,-1));

	QTableView::event(e);
	return e->isAccepted();
}




/****************************************
  #######################################
  #######################################

    THIS CLASS PROVIDES A WIDGET THAT IS
     ABLE TO RECONIZE KEY COMBINATIONS

  #######################################
  #######################################
  ***************************************/

Key::Key(QDialog *parent) : QDialog(parent) {

	// Set window title
	this->setWindowTitle(tr("Detect Keys"));

	// The appearence of the widget
	QVBoxLayout *layout = new QVBoxLayout;
	this->setLayout(layout);
	this->setFixedWidth(150);
	this->setFixedHeight(80);
	this->setWindowFlags(this->windowFlags() | Qt::CustomizeWindowHint | Qt::WindowStaysOnTopHint);

	// If a key combo already exists, then this variable ensures, that the "key" widget is re-focussed
	everythingsokay = 1;

	newkey = "";

	// The detection label showing the detected keys
	det = new QLabel(tr("Press Keys"));
	det->setAlignment(Qt::AlignCenter);
	layout->addWidget(det);

	// The Label needs to be focussed (not the button), otherwise some keys couldn't be detected (like arrows, space, ...)
	det->setFocus();

	// A click on here cancels the operation
	QPushButton *cancel = new QPushButton(tr("Cancel"));
	layout->addWidget(cancel);
	connect(cancel, SIGNAL(clicked()), this, SLOT(quitNoKeys()));

}

// Re-Enable the layout of the settings window and close
void Key::quitNoKeys() {

	qDebug() << "Exiting without new key combination";

	emit gotKeys("");
	this->close();
}

// A pressed key
void Key::keyPressEvent(QKeyEvent *e) {

	newkey = "";

	qDebug() << "Setup: Got Key Combination";

	// Get the modifier key (if there is one)


	// Check for modifiers
	if(e->modifiers().testFlag(Qt::ControlModifier))
		newkey += "Ctrl+";
	if(e->modifiers().testFlag(Qt::AltModifier))
		newkey += "Alt+";
	if(e->modifiers().testFlag(Qt::ShiftModifier))
		newkey += "Shift+";

	// Get the second key pressed
	if(e->key() == Qt::Key_Escape)
		newkey += "Escape";
	else if(e->key() == Qt::Key_Right)
		newkey += "Right";
	else if(e->key() == Qt::Key_Left)
		newkey += "Left";
	else if(e->key() == Qt::Key_Up)
		newkey += "Up";
	else if(e->key() == Qt::Key_Down)
		newkey += "Down";
	else if(e->key() == Qt::Key_Space)
		newkey += "Space";
	else if(e->key() == Qt::Key_Delete)
		newkey += "Delete";
	else if(e->key() == Qt::Key_Home)
		newkey += "Home";
	else if(e->key() == Qt::Key_End)
		newkey += "End";
	else if(e->key() == Qt::Key_PageUp)
		newkey += "PgUp";
	else if(e->key() == Qt::Key_PageDown)
		newkey += "PgDown";
	else if(e->key() == Qt::Key_Insert)
		newkey += "Insert";
	else if(e->key() == Qt::Key_Tab)
		newkey += "Tab";
	else if(e->key() != 16777251 && e->key() != 16777248 && e->key() != 16777249)
		newkey += e->key();

	// If for some reason an empty string is passed on, show a quotation mark (should never happen)
	if(newkey == "")
		newkey = "?";

	// Set key combo
	det->setText(newkey);

	// If just a normal key or a modifier+normal key is pressed: emit key combo and exit
	if(e->key() != 16777251 && e->key() != 16777248 && e->key() != 16777249) {

		// Emit Key Combo
		emit gotKeys(newkey);

		// Exit if key combo didn't exist (checked in main program above)
		if(everythingsokay)
			this->close();
		else {
			everythingsokay = 1;
			this->activateWindow();
			det->setFocus();
			det->setText(tr("Press Keys"));
		}
	}

}

// When the widget is closed manually, we need to re-enable the seetings window layout
void Key::closeEvent(QCloseEvent *e) {
	emit gotKeys("");
	e->accept();
}
