#include "tabthumbnail.h"

// Tab for adjusting Thumbnail settings
Thumbnail::Thumbnail(QWidget *parent) : QWidget(parent) {

	// Opening the thumbnail database
	db = QSqlDatabase::addDatabase("QSQLITE","thb");
	db.setDatabaseName(QDir::homePath() + "/.photo/thumbnails");

	// The main layout and scroll area
	QVBoxLayout *centralLay = new QVBoxLayout;
	QScrollArea *centralScroll = new QScrollArea;
	QVBoxLayout *central = new QVBoxLayout(centralScroll);
	QWidget *centralScrollWidget = new QWidget(centralScroll->widget());
	centralScrollWidget->setSizePolicy(QSizePolicy::Preferred,QSizePolicy::Expanding);
	centralScrollWidget->setLayout(central);
	centralScroll->setWidget(centralScrollWidget);
	centralScrollWidget->show();
	centralScroll->setWidgetResizable(true);
	centralScroll->setVerticalScrollBarPolicy(Qt::ScrollBarAsNeeded);
	centralLay->addWidget(centralScroll);
	this->setLayout(centralLay);

	// The title of the tab
	QLabel *title = new QLabel("<center><h1>" + tr("Thumbnail Settings") + "</h1></center>");
	central->addSpacing(10);
	central->addWidget(title);
	central->addSpacing(25);

	// Option to change thumbnail size
	QLabel *thumbSizeLabel = new QLabel("<b><span style=\"font-size: 12pt\">" + tr("Thumbnail Size") + "</span></b> " + tr("(Change requires restart of photo)") + "<hr>" + tr("Here you can adjust the thumbnail size. You can set it to any size between 20 and 256 pixel. Per default it is set to 80 pixel, but with different screen resolutions it might be nice to have them larger/smaller."));
	thumbSizeLabel->setWordWrap(true);
	QHBoxLayout *thumbSizeLay = new QHBoxLayout;
	thumbSizeSlider = new QSlider;
	thumbSizeSlider->setMinimum(20);
	thumbSizeSlider->setMaximum(256);
	thumbSizeSlider->setOrientation(Qt::Horizontal);
	thumbSizeSlider->setFixedWidth(150);
	thumbSizeSpin = new QSpinBox;
	thumbSizeSpin->setMinimum(20);
	thumbSizeSpin->setMaximum(256);
	thumbSizeSlider->setValue(80);
	thumbSizeSpin->setValue(80);
	thumbSizeLay->addStretch();
	thumbSizeLay->addWidget(thumbSizeSlider);
	thumbSizeLay->addWidget(thumbSizeSpin);
	thumbSizeLay->addStretch();
	central->addWidget(thumbSizeLabel);
	central->addSpacing(5);
	central->addLayout(thumbSizeLay);
	central->addSpacing(20);
	// The slider and spinbox are connected to each other
	connect(thumbSizeSlider, SIGNAL(valueChanged(int)), thumbSizeSpin, SLOT(setValue(int)));
	connect(thumbSizeSpin, SIGNAL(valueChanged(int)), thumbSizeSlider, SLOT(setValue(int)));

	// Option to move thumbnail bar to the top
	QLabel *thbPosLabel = new QLabel("<b><span style=\"font-size: 12pt\">" + tr("Change Thumbnail Position") + "</span></b> " + tr("(Change requires restart of photo)") + "<hr>" + tr("Per default the bar with the thumbnails is shown at the bottom. But some might find it nice and handy to have the thumbnail bar at the top, so that's what can be enabled here. But please note, that although it's working perfectly fine, it isn't yet the prettiest, design-wise. It'll most certainly experience some improvement in the next release."));
	thbPosLabel->setWordWrap(true);
	QHBoxLayout *thbPosLay = new QHBoxLayout;
	thbPosTop = new QRadioButton(tr("Show Thumbnails on top"));
	thbPosBot = new QRadioButton(tr("Show Thumbnails on bottom"));
	QButtonGroup *thbPosGroup = new QButtonGroup;
	thbPosGroup->addButton(thbPosTop);
	thbPosGroup->addButton(thbPosBot);
	thbPosLay->addStretch();
	thbPosLay->addWidget(thbPosTop);
	thbPosLay->addWidget(thbPosBot);
	thbPosLay->addStretch();
	central->addWidget(thbPosLabel);
	central->addSpacing(5);
	central->addLayout(thbPosLay);
	central->addSpacing(20);


	// Thumbnail cache switch
	QLabel *thumbCacheLabel = new QLabel("<b><span style=\"font-size:12pt\">" + tr("Thumbnail Cache") + "</span></b><hr>" + tr("Thumbnails can be cached in two different ways:<br>1) File Caching (according to the freedesktop.org standard) or<br>2) Database Caching (better performance and management, default option).") + "<br><br>" + tr("Both ways have their advantages and disadvantages:<br>File Caching is done according to the freedesktop.org standard and thus different applications can share the same thumbnail for the same image file. However, it's not possible to check for obsolete thumbnails (may lead to many unneeded thumbnail files).<br>Database Caching doesn't have the advantage of sharing thumbnails with other applications (and thus every thumbnails has to be newly created for Photo), but it brings a slightly better performance, and it allows a better handling of existing thumbnails (e.g. deleting obsolete thumbnails).") + "<br><br>" + tr("Photo works with either option, though the second way is set as default.") + "<br><br>" + tr("Although everybody is encouraged to use at least one of the two options, caching can be completely disabled altogether. However, that does affect the performance and usability of Photo (thumbnails have to be newly re-created every time they are needed)."));
	thumbCacheLabel->setWordWrap(true);
	QHBoxLayout *thumbCacheLay = new QHBoxLayout;
	thumbCache = new QCheckBox(tr("Enable Thumbnail Cache"));
	thumbCacheLay->addStretch();
	thumbCacheLay->addWidget(thumbCache);
	thumbCacheLay->addStretch();

	// Radiobuttons for selecting cache type
	QHBoxLayout *cacheOption = new QHBoxLayout;
	QButtonGroup *thbCacheGroup = new QButtonGroup;
	cacheFile = new QRadioButton(tr("File caching"));
	cacheDatabase = new QRadioButton(tr("Database caching"));
	thbCacheGroup->addButton(cacheFile);
	thbCacheGroup->addButton(cacheDatabase);
	cacheOption->addStretch();
	cacheOption->addWidget(cacheFile);
	cacheOption->addWidget(cacheDatabase);
	cacheOption->addStretch();
	cacheDatabase->setChecked(true);
	central->addWidget(thumbCacheLabel);
	central->addSpacing(5);
	central->addLayout(thumbCacheLay);
	central->addSpacing(5);
	central->addLayout(cacheOption);
	connect(thumbCache, SIGNAL(toggled(bool)), cacheFile, SLOT(setEnabled(bool)));
	connect(thumbCache, SIGNAL(toggled(bool)), cacheDatabase, SLOT(setEnabled(bool)));

	// We show some info about the database
	QHBoxLayout *dbInfoLay = new QHBoxLayout;
	dbInfo = new QLabel;
	dbInfoLay->addStretch();
	dbInfoLay->addWidget(dbInfo);
	dbInfoLay->addStretch();
	central->addLayout(dbInfoLay);

	// The database can be cleaned and cleared
	emptyDatabase = new QPushButton(tr("Clear Database"));
	cleanDatabase = new QPushButton(tr("Clean up Database"));
	QHBoxLayout *dbLay = new QHBoxLayout;
	dbLay->addStretch();
	dbLay->addWidget(emptyDatabase);
	dbLay->addWidget(cleanDatabase);
	dbLay->addStretch();

	central->addSpacing(10);
	central->addLayout(dbLay);
	central->addSpacing(10);

	connect(thumbCache, SIGNAL(toggled(bool)), emptyDatabase, SLOT(setEnabled(bool)));
	connect(thumbCache, SIGNAL(toggled(bool)), cleanDatabase, SLOT(setEnabled(bool)));
	connect(cacheDatabase, SIGNAL(toggled(bool)), emptyDatabase, SLOT(setEnabled(bool)));
	connect(cacheDatabase, SIGNAL(toggled(bool)), cleanDatabase, SLOT(setEnabled(bool)));

	connect(cleanDatabase, SIGNAL(clicked()), this, SLOT(doCleanDatabase()));
	connect(emptyDatabase, SIGNAL(clicked()), this, SLOT(doClearDatabase()));

	central->addStretch();
	setDatabaseInfo();


}

// We show some database information (filesize and # entries)
void Thumbnail::setDatabaseInfo() {

	if(!db.open())
		qDebug() << "ERROR: Cannot open thumbnails database:" << db.lastError().text().trimmed();
	QSqlQuery query(db);
	query.exec("SELECT COUNT(filepath) FROM Thumbnails");
	if(query.lastError().text().trimmed().length())
		qDebug() << "ERROR: (Count)" << query.lastError().text().trimmed();
	query.next();

	dbInfo->setText(tr("Current database filesize: %1 KB").arg(QFileInfo(QDir::homePath() + "/.photo/thumbnails").size()/1024) + "<br>" + tr("Entries in database: %2").arg(query.value(0).toInt()));

	query.clear();
	db.close();

}

// Clean the database (i.e. remove obsolete entries and VACUUM database)
void Thumbnail::doCleanDatabase() {

	int msg = QMessageBox::warning(this,tr("Clean Database?"),tr("You are attempting to clean up the thumbnail database (i.e. deleting obsolete thumbnails). Are you sure you want to continue? This might take a little while..."),QMessageBox::Yes | QMessageBox::Default, QMessageBox::No);

	if(msg == QMessageBox::Yes) {

		if(!db.open())
			qDebug() << "ERROR: Cannot open thumbnails database:" << db.lastError().text().trimmed();
		else {

			QSqlQuery query(db);
			query.exec("SELECT * FROM Thumbnails");

			db.transaction();

			while(query.next()) {
				QString path = query.value(query.record().indexOf("filepath")).toString();
				int mtime = query.value(query.record().indexOf("filelastmod")).toInt();

				if(!QFile(path).exists() || mtime != int(QFileInfo(path).lastModified().toTime_t())) {

					QSqlQuery query2(db);
					query2.prepare("DELETE FROM Thumbnails WHERE filepath=:path AND filelastmod=:mod");
					query2.bindValue(":mod",mtime);
					query2.bindValue(":path",path);
					query2.exec();
					if(query2.lastError().text().trimmed().length())
						qDebug() << "ERROR:" << query2.lastError().text().trimmed();
					query2.clear();

				}

			}

			db.commit();

			query.clear();
			query.exec("VACUUM");
			if(query.lastError().text().trimmed().length())
				qDebug() << "ERROR: (Vacuum)" << query.lastError().text().trimmed();
			query.clear();

		}

		db.close();

		// Update database info
		setDatabaseInfo();

	}

}

// Clear complete database (will lead to a completely empty database!!)
void Thumbnail::doClearDatabase() {

	int msg = QMessageBox::warning(this,tr("Clear Database?"),tr("You are about to clear the entire thumbnail database!! All thumbnails have to be newly re-created afterwards. Are you sure you want to continue? This step cannot be reversed!"),QMessageBox::Yes | QMessageBox::Default, QMessageBox::No);

	if(msg == QMessageBox::Yes) {

		if(!db.open())
			qDebug() << "ERROR: Cannot open thumbnails database:" << db.lastError().text().trimmed();
		else {

			QSqlQuery query(db);
			query.exec("DELETE FROM Thumbnails");
			query.clear();
			query.exec("VACUUM");
			query.clear();

		}

		db.close();

		// Update database info
		setDatabaseInfo();

	}

}

Thumbnail::~Thumbnail() { }
