#ifndef THUMBNAIL_H
#define THUMBNAIL_H

#include "thbthread.h"

#include <QGraphicsView>
#include <QGraphicsPixmapItem>
#include <QMouseEvent>
#include <QTextDocument>
#include <QScrollBar>

// The thumbnail row at the bottom
class Thumbnails : public QGraphicsView {

	Q_OBJECT

private:
	void updateGeometry();

public:
	Thumbnails();
	~Thumbnails();

	// Scene for the thumbnails
	QGraphicsScene scene;

	// Thread creating the thumbnails in the background
	ThumbThread *thread;

	// Diffferent variables
	int counttot;
	int height;
	int previousHover;
	int currentHover;
	bool cacheEnabled;

	// The path of the current file
	QString currentfile;

	// Storring info about images in current directory
	QFileInfoList allimgs;
	QStringList allThumbs;

	// Initialising the QLists for the QPixmaps(item)
	QList<QGraphicsPixmapItem*> allPixItem;
	QList<QPixmap> allPixImg;
	QList<QPixmap> allPixSel;
	QList<QPixmap> allPixHov;
	QList<QImage> allImgThumb;

	// Gathering information about all images in current directory
	void allImgs();

	// The total rotation of each thumbnail
	QList<int> totalRotation;

	// This boolean can block all functions
	bool blockAllFunction;

public slots:
	// Add a thumbnail to the scene
	void addThumbnail(QImage,QString,int);

	// Hide/Show thumbnail view
	void toggle();

	// Stop the thumbnail creation
	void stopThread();

	// Update the hovered/selected thumbnail items
	void updImgMark(int oldpos, int newpos);
	void updImgHov();

	// (Re-)Loading the thumbnails
	void loadThumbs();

	// Enable/disable the menu item "Interrupt Thumbnail creation"
	void enableMenuInterrupt();

	// Rotate a thumbnail
	void rotateThumbnail(int pos, int dir);

	void gotoFirstLast(QString which);

signals:
	// Emitted at click on thumbnail
	void loadImg(QString, int);

	// Enable/disable the menu item "Interrupt Thumbnail creation"
	void disEnableMenuInterrupt(bool);

	void resizedTo(int width, int height);

protected:
	// Detecting mouse clicks
	void mouseReleaseEvent(QMouseEvent*);
	// Detecting mouse moves
	void mouseMoveEvent(QMouseEvent *event);
	bool event(QEvent *event);
	void wheelEvent(QWheelEvent *event);

	void resizeEvent(QResizeEvent *);

};

#endif // THUMBNAIL_H
